/*
 * Copyright (C) 1996 Michael R. Elkins <me@cs.hmc.edu>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "mutt.h"
#include "mutt_curses.h"
#include "keymap.h"

#include <string.h>
#include <ctype.h>

int mutt_check_menu (const char *s)
{
  if (strcmp(s, "main-menu") == 0)
    return(MENU_MAIN);
  else if (strcmp(s, "internal-pager") == 0)
    return(MENU_PAGER);
  else if (strcmp(s, "url-menu") == 0)
    return(MENU_URL);
  else if (strcmp(s, "send-attach-menu") == 0)
    return(MENU_SENDATTACH);
  else if (strcmp(s, "send-menu") == 0)
    return(MENU_SEND);
  else if (strcmp(s, "attach-menu") == 0)
    return(MENU_ATTACH);
  else if (strcmp(s, "folder-menu") == 0)
    return(MENU_FOLDER);
  else if (strcmp(s, "alias-menu") == 0)
    return(MENU_ALIAS);
  else
    return(-1);
}

int mutt_check_key (const char *s)
{
  if (s[0] == '\\' && (s[1] == 'c' || s[1] == 'C')) {
    s += 2;
    return(toupper(*s) - '@');
  }
  if (strcmp(s, "\\n") == 0)
    return('\n');
  else if (strcmp(s, "\\r") == 0)
    return('\r');
  else if (strcmp(s, "\\t") == 0)
    return('\t');
  else if (strcasecmp(s, "home") == 0)
    return(KEY_HOME);
  else if (strcasecmp(s, "end") == 0)
    return(KEY_END);
  else if (strcasecmp(s, "pageup") == 0)
    return(KEY_PPAGE);
  else if (strcasecmp(s, "pagedown") == 0)
    return(KEY_NPAGE);
  else if (strcasecmp(s, "up") == 0)
    return(KEY_UP);
  else if (strcasecmp(s, "down") == 0)
    return(KEY_DOWN);
  else if (strcasecmp(s, "right") == 0)
    return(KEY_RIGHT);
  else if (strcasecmp(s, "left") == 0)
    return(KEY_LEFT);
  else if (strlen(s) == 1)
    return( *s);
  return(-1);
}

/*
 * bind '<key_sequence>' function-name menu-name
 */
int ci_parse_bind (char *s, char *errmsg)
{
  int count = 0, menu = -1, i;
  char *op = 0;
  int key = -1;
  struct binding_t *bindings = 0;

  while ((s = strtokq(s, " \t"))) {
    switch (count) {
    case 0:
      if ((key = mutt_check_key(s)) == -1) {
        sprintf(errmsg, "%s: unknown key", s);
        return(-1);
      }
      break;

    case 1:
      op = s;
      break;

    case 2:
      if ((menu = mutt_check_menu(s)) == -1) {
        sprintf(errmsg, "%s: no such menu", s);
        return(-1);
      }
      switch(menu) {
      case MENU_MAIN:
	bindings = Op_main;
	break;
      case MENU_SEND:
	bindings = Op_send;
	break;
      case MENU_PAGER:
	bindings = Op_pager;
	break;
      default:
	break;
      }
      break;
    }

    count++;
    s = NULL;
  }

  /*
   * Check to make sure that we got enough arguments to proceed.
   */
  if (key == -1 || !op || menu == -1) {
    strcpy(errmsg, "too few arguments to bind");
    return(-1);
  }

  if (menu == MENU_PAGER || menu == MENU_MAIN || menu == MENU_SEND) {
    /*
     * These menus have keymaps of their own and do not refer to the
     * generic map.
     */
    for (i = 0; bindings[i].name; i++)
      if (strcmp(op, bindings[i].name) == 0) {
	keymap_bind(key, menu, bindings[i].op);
	return(0);
      }
  }
  else {
    /*
     * First check the "generic" list of commands.
     */
    for (i=0; Op_generic[i].name; i++) {
      if (strcmp(op, Op_generic[i].name) == 0) {
	keymap_bind(key, menu, Op_generic[i].op);
	return(0);
      }
    }

    /*
     * Now check the menu-specific list of commands (if they exist).
     */
    if (bindings)
      for (i=0; bindings[i].name; i++)
	if (strcmp(op, bindings[i].name) == 0) {
	  keymap_bind(key, menu, bindings[i].op);
	  return(0);
	}
  }

  sprintf(errmsg, "%s: no such function in map", op);
  return(-1);
}
