/*
 * Copyright (C) 1996 Michael R. Elkins <me@cs.hmc.edu>
 * 
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 * 
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */ 

#define COLOR_C 1

#include "mutt.h"
#include "mutt_curses.h"

#include <string.h>

struct index_object {
  char *name;
  short value;
};

#ifdef HAVE_COLOR
static struct index_object colors[] = {
  { "black", COLOR_BLACK },
  { "blue", COLOR_BLUE },
  { "cyan", COLOR_CYAN },
  { "green", COLOR_GREEN },
  { "magenta", COLOR_MAGENTA },
  { "red", COLOR_RED },
  { "white", COLOR_WHITE },
  { "yellow", COLOR_YELLOW },
  { 0, 0 }
};
#endif /* HAVE_COLOR */

static struct index_object fields[] = {
  { "header",     MT_COLOR_HEADER },
  { "quoted",     MT_COLOR_QUOTED },
  { "signature",  MT_COLOR_SIGNATURE },
  { "subject",    MT_COLOR_SUBJECT },
  { "from",       MT_COLOR_FROM },
  { "toolbar",    MT_COLOR_TOOLBAR },
  { "indicator",  MT_COLOR_INDICATOR },
  { "attachment", MT_COLOR_ATTACHMENT },
  { "status",     MT_COLOR_STATUS },
  { "tree",       MT_COLOR_TREE },
  { "error",      MT_COLOR_ERROR },
  { "normal",     MT_COLOR_NORMAL },
  { 0, 0 }
};

#ifdef HAVE_COLOR
static int lookup_color (char *s)
{
  int i;

  for (i=0; colors[i].name; i++) {
    if (strcasecmp(s, colors[i].name) == 0) return(colors[i].value);
  }
  return(-1);
}
#endif /* HAVE_COOR */

void ci_init_color()
{
  int i;

  for (i=0; i<MT_COLOR_MAX; i++) {
#ifdef HAVE_COLOR
    Color_defs[i].fore = -1;
    Color_defs[i].back = -1;
    Color_defs[i].bright = 0;
    Color_defs[i].color = A_NORMAL;
#endif /* HAVE_COLOR */
    Color_defs[i].mono = A_NORMAL;
  }

  /* set mono defaults */
  Color_defs[MT_COLOR_TOOLBAR-1].mono = A_REVERSE;
  Color_defs[MT_COLOR_STATUS-1].mono = A_REVERSE;
  Color_defs[MT_COLOR_INDICATOR-1].mono = A_REVERSE;

#ifdef HAVE_COLOR
  /* set color defaults */
  Color_defs[MT_COLOR_TOOLBAR-1].color = A_REVERSE;
  Color_defs[MT_COLOR_STATUS-1].color = A_REVERSE;
  Color_defs[MT_COLOR_INDICATOR-1].color = A_REVERSE;
#endif
}

#ifdef HAVE_COLOR
void ci_start_color()
{
  int i;

  start_color();
  for (i=0; i<MT_COLOR_MAX; i++) {
    if (Color_defs[i].fore != -1) {
      init_pair(i+1, Color_defs[i].fore, Color_defs[i].back);
      Color_defs[i].color = COLOR_PAIR((i+1)) | (Color_defs[i].bright ? A_BOLD : 0);
    }
  }
}

int ci_parse_color (char *s, char *errmsg)
{
  int count = 0, object = 0, i;

  while ((s = strtok(s, " \t")) != NULL) {
    switch (count) {
    case 0:
      for (i=0; fields[i].name; i++) {
	if (strcasecmp(s, fields[i].name) == 0)
	  break;
      }
      if (i == MT_COLOR_MAX-1) {
	sprintf(errmsg, "%s: no such object", s);
	return(-1);
      }
      object = fields[i].value - 1;
      break;
    case 1:
      if (strncasecmp(s, "bright", 6) == 0) {
	Color_defs[object].bright = 1;
	s += 6;
      }
      if ((Color_defs[object].fore = lookup_color(s)) == -1) {
	sprintf(errmsg, "%s: no such color", s);
	return(-1);
      }
      break;
    case 2:
      if ((Color_defs[object].back = lookup_color(s)) == -1) {
	sprintf(errmsg, "%s: no such color", s);
	return(-1);
      }
      break;
    }
    count++;
    s = NULL;
  }
  return 0;
}
#endif /* HAVE_COLOR */

/*
 * command: mono <object> <attribute>
 *
 * set attribute for an object when using a terminal with no color support
 */
int ci_parse_mono (char *s, char *err, size_t errlen)
{
  int i;
  int object;

  if ((s = strtok(s, " \t")) == (char *)0) return(0); /* empty statement? */
  for (i=0; fields[i].name; i++) {
    if (strcasecmp(s, fields[i].name) == 0) break;
  }
  if (i == MT_COLOR_MAX - 1) {
    snprintf(err, errlen, "%s: no such attribute", s);
    return(-1);
  }
  object = fields[i].value - 1;
  while ((s = strtok((char *)0, " \t")) != (char *)0) {
    if (strcasecmp(s, "bold") == 0)
      Color_defs[object].mono = A_BOLD;
    else if (strcasecmp(s, "underline") == 0)
      Color_defs[object].mono = A_UNDERLINE;
    else if (strcasecmp(s, "none") == 0)
      Color_defs[object].mono = A_NORMAL;
    else if (strcasecmp(s, "reverse") == 0)
      Color_defs[object].mono = A_REVERSE;
    else {
      snprintf(err, errlen, "%s: no such attribute", s);
      return(-1);
    }
    s = 0;
  }
  return(0);
}
