/*
 * Copyright (C) 1996 Michael R. Elkins.
 * 
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 * 
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */ 

#include "mutt.h"
#include "rfc822.h"

#include <string.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdlib.h>

void ci_edit_headers (const char *body, ENVELOPE **env)
{
  char combined[_POSIX_PATH_MAX]; /* tempfile used to edit headers + body */
  FILE *ifp, *ofp;
  char buffer[1024];
  int i;
  ENVELOPE *n;
  time_t mtime;
  struct stat st;

  mutt_mktemp(combined);

  if ((ofp = fopen(combined, "w")) == NULL) {
    dprint(1,(debugfile, "edit_headers(): %s: couldn't open for writing!\n",
              combined));
    mutt_error("Could not open temp file!");
    return;
  }

  mutt_write_rfc822_header(ofp, *env, NULL, 1);
  fputc('\n', ofp); /* tie off the header. */

  /* now copy the body of the message. */
  ifp = fopen(body, "r");
  while ((i = fread (buffer, 1, sizeof (buffer), ifp)) > 0)
    fwrite (buffer, 1, i, ofp);

  fclose(ifp);
  fclose(ofp);

  stat(combined, &st);
  mtime = st.st_mtime;
  ci_edit_file(combined);
  stat(combined, &st);
  if (mtime == st.st_mtime) {
    dprint(1, (debugfile, "ci_edit_headers(): temp file was not modified.\n"));
    /* the file has not changed! */
    mutt_unlink(combined);
    return;
  }

  mutt_unlink(body);
  mutt_free_list(&(*env)->userhdrs);

  /* Read the temp file back in */
  ifp = fopen(combined, "r");
  ofp = fopen(body, "w");
  n = read_rfc822_header(ifp, NULL);
  while ((i = fread(buffer, 1, sizeof (buffer), ifp)) > 0)
    fwrite(buffer, 1, i, ofp);
  fclose(ofp);
  fclose(ifp);
  mutt_unlink(combined);

  /* restore old info. */
  n->references = (*env)->references;
  (*env)->references = NULL;
  mutt_free_envelope(env);
  *env = n;

  (*env)->to = mutt_expand_aliases((*env)->to);
  (*env)->cc = mutt_expand_aliases((*env)->cc);
  (*env)->bcc = mutt_expand_aliases((*env)->bcc);

  if ((*env)->from->next) {
    /*
     * User added a From: line, so remove the auto-generated one.
     */
    ADDRESS *t = (*env)->from;

    (*env)->from = (*env)->from->next;
    t->next = 0;
    mutt_free_address(&t);
  }
  
}
