/*
 * Copyright (C) 1996 Michael R. Elkins <me@cs.hmc.edu>
 * 
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 * 
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */ 

#include "mutt.h"
#include "rfc822.h"
#include "rfc1522.h"
#include "sort.h"

#include <pwd.h>
#include <ctype.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/utsname.h>
#include <stdio.h>
#include <time.h>

#define DT_BOOL 1
#define DT_STR 2
#define DT_NUM 3

#define NIL 0

struct rc_option {
  char *option;
  short type;
  void *data;
  short bit; /* DT_BOOL only */
} rc_vars[] = {
  { "alias_file",   DT_STR, AliasFile, 0 },
  { "allow_8bit",   DT_BOOL, 0, OPTALLOW8BIT },
  { "arrow_cursor", DT_BOOL, 0, OPTARROWCURSOR },
  { "askbcc",       DT_BOOL, 0, OPTASKBCC },
  { "askcc",        DT_BOOL, 0, OPTASKCC },
  { "askcopy",      DT_BOOL, 0, OPTASKCOPY },
  { "askdelete",    DT_BOOL, 0, OPTASKDEL },
  { "askmove",      DT_BOOL, 0, OPTASKMOVE },
  { "askpostpone",  DT_BOOL, 0, OPTASKPOSTPONE },
  { "askprint",     DT_BOOL, 0, OPTASKPRINT },
  { "askrecall",    DT_BOOL, 0, OPTASKRECALL },
  { "askreplyto",   DT_BOOL, 0, OPTASKREPLYTO },
  { "attribution",  DT_STR,  Attribution , 0 },
  { "autoedit",     DT_BOOL, 0, OPTAUTOEDIT },
  { "autoinclude",  DT_BOOL, 0, OPTAUTOINCLUDE },
  { "charset",      DT_STR,  Charset, 0 },
  { "confirmappend", DT_BOOL, 0, OPTCONFIRMAPPEND },
  { "confirmcreate", DT_BOOL, 0, OPTCONFIRMCREATE },
  { "confirmfiles", DT_BOOL, 0, OPTCONFIRMFILES },
  { "confirmfolders", DT_BOOL, 0, OPTCONFIRMFOLDERS },
  { "domain",       DT_STR,  Domain, 0 },
#ifdef USE_DSN
  { "dsn_notify",   DT_STR, DsnNotify, 0 },
  { "dsn_return",   DT_STR, DsnReturn, 0 },
#endif
  { "edit_hdrs",    DT_BOOL, 0, OPTEDITHDRS },
  { "editor",       DT_STR,  Editor },
  { "folder",       DT_STR,  Maildir },
  { "force_name",   DT_BOOL, 0, OPTFORCENAME },
  { "hdr_format",   DT_STR, Hdr_format, 0 },
  { "header",       DT_BOOL, 0, OPTHEADER },
  { "hold",         DT_BOOL, 0, OPTHOLD },
  { "in_reply_to",  DT_STR, In_reply_to, 0 },
  { "indent_prefix", DT_STR, Prefix, 0 },
  { "keep_empty",   DT_BOOL, 0, OPTKEEPEMPTY },
  { "localsignature", DT_STR, Local_sig, 0 },
  { "mark_old",     DT_BOOL, 0, OPTMARKOLD },
  { "mbox",         DT_STR, Inbox, 0 },
  { "metoo",        DT_BOOL, 0, OPTMETOO },
  { "pager",        DT_STR, Pager, 0 },
  { "pager_fmt",    DT_STR, Pager_fmt, 0 },
  { "pager_stop",   DT_BOOL, 0, OPTPAGERSTOP },
#ifdef _PGPPATH
  { "pgp",          DT_STR, Pgp, 0 },
  { "pgp_autosign", DT_BOOL, 0, OPTPGPAUTOSIGN },
#endif /* _PGPPATH */
  { "pointnew",     DT_BOOL, 0, OPTPOINTNEW },
  { "popdelete",    DT_BOOL, 0, OPTPOPDELETE },
  { "pophost",      DT_STR, PopHost, 0 },
  { "popuser",      DT_STR, PopUser, 0 },
  { "post_indent_str", DT_STR, PostIndentString, 0 },
  { "postponed",    DT_STR, Postponed, 0 },
  { "print",        DT_STR, Print, 0 },
  { "prompt_after", DT_BOOL, 0, OPTPROMPTAFTER },
  { "quote_regexp", DT_STR, QuoteString, 0 },
  { "readmsginc",   DT_NUM, &ReadMsginc, 0 },
  { "realname",     DT_STR, Realname, 0 },
  { "record",       DT_STR, Outbox, 0 },
  { "remotesignature", DT_STR, Remote_sig, 0 },
  { "reverse_alias", DT_BOOL, 0, OPTREVALIAS },
  { "sendmail",     DT_STR, Sendmail, 0 },
  { "shell",        DT_STR, Shell, 0 },
  { "sigdashes",    DT_BOOL, 0, OPTSIGDASHES },
  { "signature",    DT_STR, Signature, 0 },
  { "spoolfile",    DT_STR, Spoolfile, 0 },
  { "status_format", DT_STR, Status, 0 },
  { "status_on_top", DT_BOOL, 0, OPTSTATUSONTOP },
  { "tochars",      DT_STR, Tochars, 0 },
  { "timeout",      DT_NUM, &Timeout, 0 },
  { "tmpdir",       DT_STR, Tempdir, 0 },
  { "web_browser",  DT_STR, Web_browser, 0 },
  { "web_xbrowser", DT_STR, Web_Xbrowser, 0 },
  { "writemsginc",  DT_NUM, &WriteMsginc, 0 },
  { "xterm_mouse",  DT_BOOL, 0, OPTXTERMMOUSE },
  { 0, 0, 0 }
};

static void set_option (int n)
{
  if (n < 32)
    OptionsA |= (1 << n);
  else
    OptionsB |= (1 << (n - 31));
}

static void unset_option (int n)
{
  if (n < 32)
    OptionsA &= ~(1 << n);
  else
    OptionsB &= ~(1 << (n - 31));
}

/*
 * given the variable ``s'', return the index into the rc_vars array which
 * matches, or -1 if the variable is not found.
 */
static int option_index (char *s)
{
  int i;

  for (i=0; rc_vars[i].option; i++) {
    if (strcmp(s, rc_vars[i].option) == 0)
      return(i);
  }
  return(-1);
}

void remove_from_list (LIST **l, char *s)
{
  LIST *p;
  LIST *last;
  
  while ((s = strtok(s, " \t"))) {
    /* ``unCMD *'' means delete all current entries */
    if (strcmp("*", s) == 0)
      mutt_free_list(l);
    else {
      p = *l;
      last = NULL;
      while (p) {
	if (strcasecmp(s, p->data) == 0) {
	  safe_free((void **)&p->data);
	  if (last)
	    last->next = p->next;
	  else
	    (*l) = p->next;
	  safe_free((void **)&p);
	}
	else {
	  last = p;
	  p = p->next;
	}
      }
    }
    s = NULL;
  }
}

static void parse_alternates (char *s)
{
  ADDRESS *end = Alternates;
  ADDRESS *newaddr = NIL;
  char *p;

  if (end)
    while (end->next) end=end->next;

  p = s;
  while ((p = strpbrk(p, " \t")) != NULL) *p = ',';
    
  rfc822_parse_adrlist(&newaddr, s, Hostname);
  if (end)
    end->next = newaddr;
  else
    Alternates = newaddr;
}

static int add_userheader (char *s, char *errmsg, int errlen)
{
  USERHEADER *new;
  char *p, buffer[STRING];

  if ((p = strchr(s, ':')) == NULL) {
    strcpy(errmsg, "invalid header line (missing colon)");
    return(-1);
  }

  *p = 0;
  p = skip_whitespace(p+1);

  new = (USERHEADER *)safe_malloc(sizeof(USERHEADER));
  new->key = safe_strdup(s);
  new->val = safe_strdup(p);
  rfc1522_encode_string (buffer, (unsigned char *)new->val, sizeof(buffer));
  safe_free((void **)&new->val);
  new->val = safe_strdup(buffer);
  new->next = NIL;

  if (User_defined_headers) {
    USERHEADER *tmp = User_defined_headers;

    while (tmp->next) tmp = tmp->next;
    tmp->next = new;
  }
  else
    User_defined_headers = new;

  return 0;
}

static int add_alias (char *s, char *errmsg, int errlen)
{
  ALIAS *new;
  char *p;

  if ((p = strpbrk(s, " \t")) == NULL) {
    strfcpy(errmsg, "alias has no address", errlen);
    return(-1);
  }

  *p = 0;
  p = skip_whitespace(p+1);

  new = (ALIAS *)safe_malloc(sizeof(ALIAS));
  new->name = safe_strdup(s);
  new->addr = 0;
  rfc822_parse_adrlist(&new->addr, p, "@");
  new->next = NIL;
  if (Aliases == NIL)
      Aliases = new;
  else {
    ALIAS *tmp = Aliases;

    while (tmp->next) tmp = tmp->next;
    tmp->next = new;
  }
  return 0;
}

void mutt_add_to_list (LIST **list, char *s)
{
  LIST *last = 0;
  LIST *t;

  while ((s = strtok(s, " \t,"))) {
    /* check to make sure the item is not already on this list */
    last = *list;
    while (last) {
      if (strcasecmp(s, last->data) == 0) {
	/* already on the list, so just ignore it */
	last = NULL;
	break;
      }
      if (!last->next) break;
      last = last->next;
    }
    if (!*list || last) {
      t = (LIST *)safe_malloc(sizeof(LIST));
      memset(t, 0, sizeof(LIST));
      t->data = safe_strdup(s);
      if (last) {
	last->next = t;
	last = last->next;
      }
      else
	*list = last = t;
    }
    s = NULL;
  }
}

/*
 * reads the specified initialization file.  returns -1 if errors were found
 * so that we can pause to let the user know...
 */
static int source_rc (char *rcfile)
{
  FILE *f;
  short line=0, rc=0;
  char buffer[LONG_STRING], errbuf[SHORT_STRING];

  mutt_expand_path(rcfile);

#ifdef USE_SETGID
  if (access(rcfile, R_OK) == -1) {
    printf("%s: permission denied.\n", rcfile);
    return(-1);
  }
#endif /* USE_SETGID */

  if ((f = fopen(rcfile, "r")) == NULL) {
    fprintf(stderr, "%s: no such file or directory.\n", rcfile);
    return(-1);
  }
  while (fgets(buffer, LONG_STRING, f) != NULL) {
    line++;
    
    if (mutt_parse_rc_line(buffer, 1, errbuf, sizeof(errbuf)) == -1) {
      printf("Error in %s, line %d: %s\n", rcfile, line, errbuf);
      rc = -1;
    }
  }
  fclose(f);
  return(rc);
}

/*
 * Takes a string, and executes any `commands` and replaces the 
 * command string with the first line of the command output
 */
static void mutt_string_cmd_sub(char *s, int slen)
{
   char *tmp;
   int incmd = 0;
   char *cmd, *resp; 
   int cmdind = 0;
   int x,y;

   dprint(1,(debugfile,"mutt_string_cmd_sub(): Substring: %s, slen: %d\n",s,slen));
   tmp = safe_strdup(s);
   cmd = (char *) safe_malloc(sizeof(char *) * slen);
   resp = (char *) safe_malloc(sizeof(char *) * slen);
   for (x = 0, y = 0; x < slen && y < slen && s[x]; x++) {
     if (incmd) {
       /* dprint(1,(debugfile,"mutt_string_cmd_sub(): cmd: %s\n",s)); */
       if (s[x] == '\\') {
	 x++;
	 if (s[x] == 'n') {
	      cmd[cmdind++] = '\n';
         } 
	 else if (s[x] == 't') {
	      cmd[cmdind++] = '\t';
         }
	 else if (s[x])
	   cmd[cmdind++] = s[x];
       }
       else if ((s[x] == '`') && (s[x-1] != '\\'))
	 incmd = 0;
       else
	 cmd[cmdind++] = s[x];

       if (!incmd) {
	 cmd[cmdind] = 0;
	 if (cmd[0]) {
	   FILE *out = NULL;
	   int rind = 0;

           dprint(1,(debugfile,"mutt_string_cmd_sub(): Command: %s\n",cmd));
	   mutt_create_filter(cmd, NULL, &out);
	   fgets(resp,slen,out);
	   fclose(out);
           dprint(1,(debugfile,"mutt_string_cmd_sub(): Response: %s\n",resp));

	   /* copy response into temp string */
	   while (resp[rind] && y < slen) {
	     if (resp[rind] != '\n')
	       tmp[y++] = resp[rind++];
	     else
	       rind++;
	   }
	   if (y < slen)
	     tmp[y] = 0;
	   else
	     tmp[slen-1] = 0;
           /* reset command */
           cmdind = 0;
         }
       }
     } else {
       if ((s[x] == '`') && (!x || (s[x-1] != '\\'))) incmd = 1;
	 else tmp[y++] = s[x];
     }
   }
   strncpy(s,tmp,slen);
   safe_free((void **)&cmd);
   safe_free((void **)&resp);
}

/*
 * if sourcing is TRUE, we are reading from an init file, otherwise assume
 * that this was entered on a command line.
 */
int mutt_parse_rc_line (char *s, int sourcing, char *errmsg, int errlen)
{
  char *p;
  int unset=0;
  int i;

  *errmsg = 0;

  s = skip_whitespace(s);

  /* Strip those comments!  Allow the comment character (#) to be escaped */
  if (*s == '#') return 0; /* nothing to do */

  p = s+1;
  while ((p = strchr(p, '#'))) {
    if (*(p-1) == '\\') {
      strcpy(p-1, p);
    }
    else {
      *p = 0;
      break;
    }
  }

  /* Remove any trailing whitespace */
  p = s + strlen(s) - 1;
  while (isspace(*p) && (p >= s)) {
    *p = 0;
    if (p == s)
      break;
    else
      p--;
  }

  /* Is there anything left on this line? */
  if (!*s) return 0;

  if ((p = strchr(s,'`')) && (*(p-1) != '\\')) 
    mutt_string_cmd_sub(s,LONG_STRING);

  if (strncmp(s, "set", 3) == 0 || (strncmp(s, "unset", 5) == 0 && (unset=1))) {
    int idx;
    char keyword[SHORT_STRING], *pk;

    /* position on the next word */
    if ((p = strpbrk(s, " \t")) == NULL) {
      strcpy(errmsg, "\"set\" requires argument");
      return(-1);
    }
    p = skip_whitespace(p);
    
    /* this is an alternate way to unset a boolean variable */
    if (strncmp("no", p, 2) == 0) {
      unset = 1;
      p += 2;
    }

    /* Get the keyword */
    pk = keyword;
    while (*p && !isspace(*p) && *p != '=') *pk++ = *p++;
    *pk = 0;
    p = skip_whitespace(p);
    
    if ((idx = option_index(keyword)) == -1) {
      snprintf(errmsg, errlen, "%s: unknown variable", keyword);
      return(-1);
    }

    if (rc_vars[idx].type == DT_BOOL) {
      if (unset)
	unset_option(rc_vars[idx].bit);
      else
	set_option(rc_vars[idx].bit);
    }
    else if (rc_vars[idx].type == DT_STR) {
      char *ptr = rc_vars[idx].data;
      
      if (*p != '=') {
	if (!sourcing && *p == 0) {
	  /* user requested the value of this variable */
	  snprintf(errmsg, errlen, "%s=\"%s\"", rc_vars[idx].option, ptr);
	  return 0;
	}
	snprintf(errmsg, errlen, "expected \"=\" but got \"%s\"", p);
	return(-1);
      }
      p = skip_whitespace(p+1);
      if (*p == '"') {
	p++;
	while (*p) {
	  if (*p == '\\') {
	    p++;
	    if (*p == 'n') {
	      *ptr++ = '\n';
              p++;
            }
	    else if (*p == 't') {
	      *ptr++ = '\t';
              p++;
            }
	    else if (*p)
	      *ptr++ = *p++;
	    else
	      break;
	  }
	  else if (*p == '\"')
	    break;
	  else
	    *ptr++ = *p++;
	}
	*ptr = 0;
      }
      else {
	while (*p && !isspace(*p)) *ptr++ = *p++;
	*ptr = 0;
      }
    }
    else if (rc_vars[idx].type == DT_NUM) {
      short *ptr = rc_vars[idx].data;
      
       if (*p != '=') {
	if (!sourcing && *p == 0) {
	  snprintf(errmsg, errlen, "%s=%d", rc_vars[idx].option, *ptr);
	  return 0;
	}
	snprintf(errmsg, errlen, "expected \"=\" but got \"%s\"", p);
	return(-1);
      }
      p = skip_whitespace(p+1);
      *ptr = (short)atoi(p);
    }
  }
  else if (strncmp(s, "alias", 5) == 0) {
    p = skip_whitespace(s+5);
    if (add_alias(p, errmsg, errlen) == -1) return(-1);
  }
  else if (strncmp(s, "alternates", 10) == 0) {
    p = skip_whitespace(s+10);
    parse_alternates(p);
  }
  else if (strncmp(s, "macro", 5) == 0) {
    if (mutt_parse_macro(skip_whitespace(s+5), errmsg, errlen) == -1)
      return(-1);
  }
  else if (strncmp(s, "bind", 4) == 0) {
    p = skip_whitespace(s+4);
    if (ci_parse_bind(p, errmsg) == -1) return(-1);
  }
  else if (strncmp(s, "color", 5) == 0) {
#ifdef HAVE_COLOR
    /* If we don't have color support, silently ignore color commands. */
    if (ci_parse_color(skip_whitespace(s+5), errmsg) == -1) return(-1);
#endif /* HAVE_COLOR */
  }
  else if (strncmp(s, "ignore", 6) == 0)
    mutt_add_to_list(&Ignore, skip_whitespace(s+6));
  else if (strncmp(s, "unignore", 8) == 0)
    remove_from_list(&Ignore, skip_whitespace(s+8));
  else if (strncmp(s, "lists", 5) == 0)
    mutt_add_to_list(&Mailing_lists, skip_whitespace(s+5));
  else if (strncmp(s, "unlists", 7) == 0)
    remove_from_list(&Mailing_lists, skip_whitespace(s+7));
  else if (strncmp(s, "localsite", 9) == 0)
    mutt_add_to_list(&Localsites, skip_whitespace(s+9));
  else if (strncmp(s, "unlocalsite", 11) == 0)
    remove_from_list(&Localsites, skip_whitespace(s+11));
  else if (strncmp(s, "mono", 4) == 0) {
    p = skip_whitespace(s+4);
    if (ci_parse_mono(p, errmsg, errlen) != 0) return(-1);
  }
  else if (strncmp(s, "my_hdr", 6) == 0) {
    p = skip_whitespace(s+6);
    if (add_userheader(p, errmsg, errlen) == -1) return(-1);
  }
  else if (strncmp(s, "sort", 4) == 0) {
    p = skip_whitespace(s+4);
    if (!*p) {
      snprintf(errmsg, errlen, "Sorting by %s", Sort_methods[(Sort & SORTMASK)-1]);
      return(0);
    }
    Sort = 0;
    if (strncasecmp(p, "reverse-", 8) == 0) {
      Sort = SORTREVERSE;
      p += 8;
    }
    for (i=0; Sort_methods[i]; i++) {
      if (strcasecmp(p, Sort_methods[i]) == 0) {
	Sort |= i+1;
	break;
      }
    }
    if (i >= SORTMAX) {
      snprintf(errmsg, errlen, "%s: unknown sorting method", p);
      Sort = SORTDATE;
      return(-1);
    }
  }
  else if (strncmp(s, "source", 6) == 0) {
    p = skip_whitespace(s+6);
    if (source_rc(p) == -1) return(-1);
  }
  else {
    if ((p = strpbrk(s, " \t")) != 0) *p = 0;
    snprintf(errmsg, errlen, "%s: unknown command", s);
    return(-1);
  }

  return 0;
}

#ifdef DEBUG
static void start_debug (void)
{
  time_t t;
  int i;
  char buf[_POSIX_PATH_MAX];
  char buf2[_POSIX_PATH_MAX];

  /* rotate the old debug logs */
  for (i=3; i>=0; i--) {
    snprintf(buf, sizeof(buf), "%s/.muttdebug%d", Homedir, i);
    snprintf(buf2, sizeof(buf2), "%s/.muttdebug%d", Homedir, i+1);
    rename(buf, buf2);
  }
  if ((debugfile = fopen(buf, "w")) != NULL) {
    t = time(0);
    fprintf(debugfile, "%s started at %s.\nDebugging at level %d.\n\n",
	    Mutt_version, asctime(localtime(&t)), debuglevel);
  }
}
#endif

int mutt_init (int skip_sys_rc)
{
  struct passwd *pw;
  char buffer[STRING];
  char *p;
  short code = 0;
#ifndef HOSTNAME
  struct utsname utsname;
#endif

  /* Get some information about the user */
  pw = getpwuid(getuid());
  strfcpy(Username, pw->pw_name, sizeof(Username));

  /*
   * on one of the systems I use, getcwd() does not return the same prefix
   * as is listed in the passwd file
   */
  if ((p = getenv("HOME")) != (char *)0)
    strfcpy(Homedir, p, sizeof(Homedir));
  else
    strfcpy(Homedir, pw->pw_dir, sizeof(Homedir));

  strfcpy(Realname, pw->pw_gecos, sizeof(Realname));
  if ((p = strchr(Realname, ',')) != 0) *p = 0;
  strfcpy(Shell, pw->pw_shell, sizeof(Shell));

#ifdef DEBUG
  /* Start up debugging mode if requested */
  if (debuglevel > 0) start_debug();
#endif

  /* And about the host... */
#ifdef HOSTNAME
  strfcpy(Hostname, HOSTNAME, sizeof(Hostname));
#else
  uname(&utsname);
  strfcpy(Hostname, utsname.nodename, sizeof(Hostname));
  /* some systems report the FQDN instead of just the hostname */
  if ((p = strchr(Hostname, '.')))
    *p = 0;
#endif /* HOSTNAME */

#ifdef USE_DOMAIN
  strfcpy(Domain, DOMAIN, sizeof(Domain));
#endif

  /* Set some defaults */

  set_option(OPTASKRECALL);
  set_option(OPTHOLD);
  set_option(OPTPROMPTAFTER);
  set_option(OPTALLOW8BIT);
  set_option(OPTPOINTNEW);
  set_option(OPTASKDEL);
  set_option(OPTASKMOVE);
  set_option(OPTASKPOSTPONE);
  set_option(OPTASKPRINT);
  set_option(OPTMARKOLD);
  set_option(OPTCONFIRMCREATE);
  set_option(OPTCONFIRMFILES);
  set_option(OPTSIGDASHES);
  
#ifdef HOMESPOOL
  snprintf(Spoolfile, sizeof(Spoolfile), "%s/%s", Homedir, MAILPATH);
#else
  if ((p = getenv("MAIL")))
    strfcpy(Spoolfile, p, sizeof(Spoolfile));
  else
    snprintf(Spoolfile, sizeof(Spoolfile), "%s/%s", MAILPATH, Username);
#endif

  strcpy(AliasFile, "~/.muttrc");
  strcpy(Attribution, "%n writes:");
  /* For HanMutt - pms@romance.kaist.ac.kr */
  strcpy(Charset, "euc-kr");
#ifdef USE_DSN
  strcpy(DsnNotify, "failure,delay");
  strcpy(DsnReturn, "hdrs");
#endif
  strcpy(Hdr_format, "%M %2N %-15.15L (%4l) %s");
  strcpy(In_reply_to, "%i; from %n on %d");
  strcpy(Inbox, "~/mbox");
  strcpy(Maildir, "~/Mail");
  strcpy(Pager, "builtin");  
  strcpy(Pager_fmt, "%S %C/%T: %-20.20n   %s");
  strcpy(Postponed, "~/postponed");
  strcpy(Prefix, "> ");
  strcpy(Print, "lpr");
  strcpy(QuoteString, "^[|>:}#] *");
  strfcpy(Sendmail, SENDMAIL, sizeof(Sendmail));
  strcpy(Signature, "~/.signature");
  Sort = SORTDATE;
  strcpy(Status, "---Mutt: %f [%?M/?%m msgs, %?n new, ?%?t tagged, ?%?p post, ?%l bytes]---(%s)----------------------------------");
  strcpy(Tempdir, "/tmp");
  strcpy(Tochars, " +TCF");
  Metamail[0] = 0;
#ifdef _PGPPATH
  strfcpy(Pgp, _PGPPATH, sizeof(Pgp));
#endif /* _PGPPATH */
  strfcpy(PopUser, Username, sizeof(PopUser));

  if ((p = getenv("VISUAL")) != NULL) {
    strfcpy(Editor, p, sizeof(Editor));
  }
  else if ((p = getenv("EDITOR")) != NULL) {
    strfcpy(Editor, p, sizeof(Editor));
  }
  else
    strcpy(Editor, "vi");

  if ((p = getenv("REPLYTO")) != NULL) {
    char error[SHORT_STRING];

    snprintf(buffer, sizeof(buffer), "Reply-To: %s", p);
    add_userheader(buffer, error, sizeof(error));
  }

  /*
   * Process the global rc file if it exists and the user hasn't explicity
   * requested not to via "-n".
   */
  if (!skip_sys_rc && access(SYSMUTTRC, F_OK) != -1) source_rc(SYSMUTTRC);

  /*
   * Read the user's initialization file.
   */
  mutt_expand_path(Muttrc);
  if (access(Muttrc, F_OK) != -1)
    code = source_rc(Muttrc);

  /*
   * Special case which needs handling up front since expand_path() is not
   * recursive.
   */
  if (Maildir[0] == '~') {
    snprintf(buffer, sizeof(buffer), "%s/%s", Homedir, Maildir+2);
    strfcpy(Maildir, buffer, sizeof(Maildir));
  }

  /* expand these now so we don't have to keep doing it over and over */
  mutt_expand_path(AliasFile);
  mutt_expand_path(Inbox);
  mutt_expand_path(Outbox);
  mutt_expand_path(Postponed);
  mutt_expand_path(Spoolfile);
  mutt_expand_path(Local_sig);
  mutt_expand_path(Remote_sig);
  mutt_expand_path(Signature);
  mutt_expand_path(Tempdir);

  /*
   * The user may have changed the domain name in an init file, so wait
   * until now to set our FQDN for outgoing mail.
   */
#ifdef USE_DOMAIN
# ifdef HIDDEN_HOST
  strfcpy(Fqdn, Domain, sizeof(Fqdn));
# else
  snprintf(Fqdn, sizeof(Fqdn), "%s.%s", Hostname, Domain);
# endif /* HIDDEN_HOST */
#else
#ifdef HIDDEN_HOST
  /*
   * "@" has special meaning to rfc822_write_address(); it refers to the
   * local host.  Setting the Fqdn to this will cause rfc822_write_address()
   * to NOT append any domain to the mailbox.
   */
  strcpy(Fqdn, "@");
#else
  strfcpy(Fqdn, Hostname, sizeof(Fqdn));
#endif
#endif

  return code;
}
