/* Copyright (C) 1996 Thomas 'Mike' Michlmayr
 * 
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 * 
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Modified by Michael Elkins <me@cs.hmc.edu>
 */ 

#define KEYMAP_C

#include "mutt.h"
#include "mutt_curses.h"
#include "keymap.h"

#include <string.h>

/* global hashtable for keys (menus are in linked lists starting here) */
KEYMAP *Keymap[MAX_KEYS];

/* keymap for function keys which are outside the normal ASCII range */
KEYMAP *Keymap_func[KEY_MAX-KEY_MIN];

/*
 * internal function used to insert a binding into the linked list
 */
void keymap_bind (int ch, int menu, int function) {
  KEYMAP *p = (ch < KEY_MIN) ? Keymap[ch] : Keymap_func[ch-KEY_MIN];
  
  if (!p) {
    p = (KEYMAP *)safe_malloc(sizeof(KEYMAP));
    p->menu = menu;
    p->function = function;
    p->next = 0;
    if (ch < KEY_MIN)
      Keymap[ch] = p;
    else
      Keymap_func[ch-KEY_MIN] = p;
    return;
  }

  /* need to get rid of any previous bindings for this key in this menu */
  
  FOREVER {
    if (p->menu == menu) {
      /* this binding overrides the previous binding. */
      p->function = function;
      return;
    }
    if (!p->next) {
      /* no binding exists, so create a new one. */
      p->next = (KEYMAP *)safe_malloc(sizeof(KEYMAP));
      p = p->next;
      p->menu = menu;
      p->function = function;
      p->next = 0;
      return;
    }
    p = p->next;
  }

  /* not reached */
}

/*
 * This function generates the default keybindings for a given menu.
 */
static void generic_bindings (int menu) {
  keymap_bind('?',  menu, OP_GENERIC_HELP);
  keymap_bind('\n', menu, OP_GENERIC_SELECT_ENTRY);
  keymap_bind('\r', menu, OP_GENERIC_SELECT_ENTRY);
  keymap_bind('j',  menu, OP_GENERIC_NEXT_ENTRY);
  keymap_bind('k',  menu, OP_GENERIC_PREVIOUS_ENTRY);
  keymap_bind('h',  menu, OP_GENERIC_PREVIOUS_PAGE);
  keymap_bind('l',  menu, OP_GENERIC_NEXT_PAGE);
  keymap_bind('q',  menu, OP_GENERIC_EXIT);
  keymap_bind('x',  menu, OP_GENERIC_EXIT);
  keymap_bind('=',  menu, OP_GENERIC_FIRST_ENTRY);
  keymap_bind('*',  menu, OP_GENERIC_LAST_ENTRY);
  keymap_bind(ctrl('L'), menu, OP_GENERIC_REFRESH);
  keymap_bind('H',  menu, OP_GENERIC_TOP_OF_PAGE);
  keymap_bind('M',  menu, OP_GENERIC_MIDDLE_OF_PAGE);
  keymap_bind('L',  menu, OP_GENERIC_BOTTOM_OF_PAGE);

  keymap_bind(KEY_NPAGE, menu, OP_GENERIC_NEXT_PAGE);
  keymap_bind(KEY_RIGHT, menu, OP_GENERIC_NEXT_PAGE);
  keymap_bind(KEY_PPAGE, menu, OP_GENERIC_PREVIOUS_PAGE);
  keymap_bind(KEY_LEFT,  menu, OP_GENERIC_PREVIOUS_PAGE);
  keymap_bind(KEY_UP,    menu, OP_GENERIC_PREVIOUS_ENTRY);
  keymap_bind(KEY_DOWN,  menu, OP_GENERIC_NEXT_ENTRY);
  keymap_bind(KEY_HOME,  menu, OP_GENERIC_FIRST_ENTRY);
}

/*
 * Initialize the keymap with the default bindings.
 */
void keymap_init (void)
{
  memset(Keymap, 0, sizeof(Keymap));
  memset(Keymap_func, 0, sizeof(Keymap_func));

  keymap_bind('a',  MENU_MAIN, OP_MAIN_ALIAS);
  keymap_bind('B',  MENU_MAIN, OP_MAIN_BOTTOM_OF_PAGE);
  keymap_bind('b',  MENU_MAIN, OP_MAIN_BOUNCE_MESSAGE);
  keymap_bind('C',  MENU_MAIN, OP_MAIN_COPY_MESSAGE);
  keymap_bind('c',  MENU_MAIN, OP_MAIN_CHANGE_FOLDER);
  keymap_bind('D',  MENU_MAIN, OP_MAIN_DELETE_PATTERN);
  keymap_bind(ctrl('D'), MENU_MAIN, OP_MAIN_DELETE_THREAD);
  keymap_bind('d',  MENU_MAIN, OP_MAIN_DELETE_MESSAGE);
  keymap_bind('j',  MENU_MAIN, OP_MAIN_NEXT_UNDELETED);
  keymap_bind('J',  MENU_MAIN, OP_MAIN_NEXT_MESSAGE);
#ifdef _PGPPATH
  keymap_bind(ctrl('F'), MENU_MAIN, OP_MAIN_FORGET_PASSPHRASE);
#endif /* _PGPPATH */
  keymap_bind('F',  MENU_MAIN, OP_MAIN_FLAG_MESSAGE);
  keymap_bind('f',  MENU_MAIN, OP_MAIN_FORWARD_MESSAGE);
#ifdef USE_POP
  keymap_bind('G',  MENU_MAIN, OP_MAIN_FETCH_MAIL);
#endif /* USE_POP */
  keymap_bind('g',  MENU_MAIN, OP_MAIN_GROUP_REPLY);
  keymap_bind('H',  MENU_MAIN, OP_MAIN_TOP_OF_PAGE);
  keymap_bind('h',  MENU_MAIN, OP_MAIN_DISPLAY_HEADERS);
  keymap_bind('k',  MENU_MAIN, OP_MAIN_PREVIOUS_UNDELETED);
  keymap_bind('K',  MENU_MAIN, OP_MAIN_PREVIOUS_MESSAGE);
  keymap_bind(ctrl('L'), MENU_MAIN, OP_MAIN_REFRESH);
  keymap_bind('L',  MENU_MAIN, OP_MAIN_LIST_REPLY);
  keymap_bind('l',  MENU_MAIN, OP_MAIN_LIMIT);
  keymap_bind('M',  MENU_MAIN, OP_MAIN_MIDDLE_OF_PAGE);
  keymap_bind('m',  MENU_MAIN, OP_MAIN_MAIL);
  keymap_bind('N',  MENU_MAIN, OP_MAIN_MARK_AS_NEW);
  keymap_bind('n',  MENU_MAIN, OP_MAIN_NEXT_UNDELETED);
  keymap_bind(ctrl('N'), MENU_MAIN, OP_MAIN_NEXT_THREAD);
  keymap_bind('o',  MENU_MAIN, OP_MAIN_SORT_MESSAGES);
  keymap_bind('p',  MENU_MAIN, OP_MAIN_PRINT_MESSAGE);
  keymap_bind(ctrl('P'), MENU_MAIN, OP_MAIN_PREVIOUS_THREAD);
  keymap_bind('Q',  MENU_MAIN, OP_MAIN_QUIT);
  keymap_bind('q',  MENU_MAIN, OP_MAIN_QUIT);
  keymap_bind('r',  MENU_MAIN, OP_MAIN_REPLY);
  keymap_bind('R',  MENU_MAIN, OP_MAIN_RECALL_MESSAGE);
  keymap_bind('s',  MENU_MAIN, OP_MAIN_SAVE_MESSAGE);
  keymap_bind('T',  MENU_MAIN, OP_MAIN_TAG_PATTERN);
  keymap_bind('t',  MENU_MAIN, OP_MAIN_TAG_MESSAGE);
  keymap_bind(ctrl('T'), MENU_MAIN, OP_MAIN_UNTAG_PATTERN);
  keymap_bind('U',  MENU_MAIN, OP_MAIN_UNDELETE_PATTERN);
  keymap_bind('u',  MENU_MAIN, OP_MAIN_UNDELETE_MESSAGE);
  keymap_bind(ctrl('U'), MENU_MAIN, OP_MAIN_BROWSE_URL);
  keymap_bind('v',  MENU_MAIN, OP_MAIN_VIEW_ATTACHMENTS);
  keymap_bind('x',  MENU_MAIN, OP_MAIN_EXIT);
  keymap_bind('z',  MENU_MAIN, OP_MAIN_NEXT_PAGE);
  keymap_bind('Z',  MENU_MAIN, OP_MAIN_PREVIOUS_PAGE);
  keymap_bind('*',  MENU_MAIN, OP_MAIN_LAST_MESSAGE);
  keymap_bind('=',  MENU_MAIN, OP_MAIN_FIRST_MESSAGE);
  keymap_bind(' ',  MENU_MAIN, OP_MAIN_DISPLAY_MESSAGE);
  keymap_bind('\n', MENU_MAIN, OP_MAIN_DISPLAY_MESSAGE);
  keymap_bind('\r', MENU_MAIN, OP_MAIN_DISPLAY_MESSAGE);
  keymap_bind('$',  MENU_MAIN, OP_MAIN_SYNC_FOLDER);
  keymap_bind('/',  MENU_MAIN, OP_MAIN_SEARCH);
  keymap_bind('|',  MENU_MAIN, OP_MAIN_PIPE_MESSAGE);
  keymap_bind('?',  MENU_MAIN, OP_MAIN_HELP);
  keymap_bind(':',  MENU_MAIN, OP_MAIN_ENTER_COMMAND);
  keymap_bind('!',  MENU_MAIN, OP_MAIN_SUBSHELL);
  keymap_bind('\t', MENU_MAIN, OP_MAIN_NEXT_NEW);
  keymap_bind('%',  MENU_MAIN, OP_MAIN_DISPLAY_ADDRESS);
  
  keymap_bind('1', MENU_MAIN, OP_MAIN_JUMP);
  keymap_bind('2', MENU_MAIN, OP_MAIN_JUMP);
  keymap_bind('3', MENU_MAIN, OP_MAIN_JUMP);
  keymap_bind('4', MENU_MAIN, OP_MAIN_JUMP);
  keymap_bind('5', MENU_MAIN, OP_MAIN_JUMP);
  keymap_bind('6', MENU_MAIN, OP_MAIN_JUMP);
  keymap_bind('7', MENU_MAIN, OP_MAIN_JUMP);
  keymap_bind('8', MENU_MAIN, OP_MAIN_JUMP);
  keymap_bind('9', MENU_MAIN, OP_MAIN_JUMP);
  
  keymap_bind(KEY_HOME,  MENU_MAIN, OP_MAIN_FIRST_MESSAGE);
  keymap_bind(KEY_END,   MENU_MAIN, OP_MAIN_LAST_MESSAGE);
  keymap_bind(KEY_RIGHT, MENU_MAIN, OP_MAIN_NEXT_PAGE);
  keymap_bind(KEY_NPAGE, MENU_MAIN, OP_MAIN_NEXT_PAGE);
  keymap_bind(KEY_LEFT,  MENU_MAIN, OP_MAIN_PREVIOUS_PAGE);
  keymap_bind(KEY_PPAGE, MENU_MAIN, OP_MAIN_PREVIOUS_PAGE);
  keymap_bind(KEY_UP,    MENU_MAIN, OP_MAIN_PREVIOUS_UNDELETED);
  keymap_bind(KEY_DOWN,  MENU_MAIN, OP_MAIN_NEXT_UNDELETED);

  /* defaults for the internal pager. */
  
  keymap_bind(' ',  MENU_PAGER, OP_PAGER_NEXT_PAGE);
  keymap_bind('-',  MENU_PAGER, OP_PAGER_PREVIOUS_PAGE);
  keymap_bind('\n', MENU_PAGER, OP_PAGER_NEXT_LINE);
  keymap_bind('\r', MENU_PAGER, OP_PAGER_NEXT_LINE);
  keymap_bind('^',  MENU_PAGER, OP_PAGER_TOP);
  keymap_bind('/',  MENU_PAGER, OP_PAGER_SEARCH);
  keymap_bind(127,  MENU_PAGER, OP_PAGER_PREVIOUS_LINE);
  keymap_bind('?',  MENU_PAGER, OP_PAGER_HELP);
  keymap_bind('$',  MENU_PAGER, OP_PAGER_BOTTOM);
  keymap_bind('T',  MENU_PAGER, OP_PAGER_HIDE_QUOTED);
  
  keymap_bind('\t', MENU_PAGER, OP_MAIN_NEXT_NEW);
  keymap_bind('b', MENU_PAGER, OP_MAIN_BOUNCE_MESSAGE);
  keymap_bind('C', MENU_PAGER, OP_MAIN_COPY_MESSAGE);
  keymap_bind('d', MENU_PAGER, OP_MAIN_DELETE_MESSAGE);
  keymap_bind('F', MENU_PAGER, OP_MAIN_FLAG_MESSAGE);
  keymap_bind('f', MENU_PAGER, OP_MAIN_FORWARD_MESSAGE);
  keymap_bind('g', MENU_PAGER, OP_MAIN_GROUP_REPLY);
  keymap_bind('h', MENU_PAGER, OP_MAIN_DISPLAY_HEADERS);
  keymap_bind('i', MENU_PAGER, OP_MAIN_EXIT);
  keymap_bind('j', MENU_PAGER, OP_MAIN_NEXT_UNDELETED);
  keymap_bind('J', MENU_PAGER, OP_MAIN_NEXT_MESSAGE);
  keymap_bind('k', MENU_PAGER, OP_MAIN_PREVIOUS_UNDELETED);
  keymap_bind('K', MENU_PAGER, OP_MAIN_PREVIOUS_MESSAGE);
  keymap_bind('L', MENU_PAGER, OP_MAIN_LIST_REPLY);
  keymap_bind('m', MENU_PAGER, OP_MAIN_MAIL);
  keymap_bind('n', MENU_PAGER, OP_MAIN_NEXT_UNDELETED);
  keymap_bind('N', MENU_PAGER, OP_MAIN_MARK_AS_NEW);
  keymap_bind('p', MENU_PAGER, OP_MAIN_PRINT_MESSAGE);
  keymap_bind('q', MENU_PAGER, OP_MAIN_EXIT);
  keymap_bind('r', MENU_PAGER, OP_MAIN_REPLY);
  keymap_bind('s', MENU_PAGER, OP_MAIN_SAVE_MESSAGE);
  keymap_bind('u', MENU_PAGER, OP_MAIN_UNDELETE_MESSAGE);
  keymap_bind(ctrl('U'), MENU_PAGER, OP_MAIN_BROWSE_URL);
  keymap_bind('x', MENU_PAGER, OP_MAIN_EXIT);
  keymap_bind('|', MENU_PAGER, OP_MAIN_PIPE_MESSAGE);

  keymap_bind(KEY_HOME, MENU_PAGER, OP_PAGER_TOP);
  keymap_bind(KEY_END, MENU_PAGER, OP_PAGER_BOTTOM);
  keymap_bind(KEY_NPAGE, MENU_PAGER, OP_PAGER_NEXT_PAGE);
  keymap_bind(KEY_PPAGE, MENU_PAGER, OP_PAGER_PREVIOUS_PAGE);
  keymap_bind(KEY_UP, MENU_PAGER, OP_MAIN_PREVIOUS_UNDELETED);
  keymap_bind(KEY_RIGHT, MENU_PAGER, OP_MAIN_NEXT_UNDELETED);
  keymap_bind(KEY_DOWN, MENU_PAGER, OP_MAIN_NEXT_UNDELETED);
  keymap_bind(KEY_LEFT, MENU_PAGER, OP_MAIN_PREVIOUS_UNDELETED);

  /* defaults for the attachment menu. */

  generic_bindings(MENU_ATTACH);
  keymap_bind('s', MENU_ATTACH, OP_ATTACH_SAVE_ENTRY);
  keymap_bind('p', MENU_ATTACH, OP_ATTACH_PRINT_ENTRY);

  /* defaults for the send menu */

  keymap_bind('?',  MENU_SEND, OP_SEND_HELP);
  keymap_bind('a',  MENU_SEND, OP_SEND_ATTACH_FILE);
  keymap_bind('b',  MENU_SEND, OP_SEND_EDIT_BCC);
  keymap_bind('c',  MENU_SEND, OP_SEND_EDIT_CC);
  keymap_bind('e',  MENU_SEND, OP_SEND_EDIT_MESSAGE);
  keymap_bind('E',  MENU_SEND, OP_SEND_EDIT_HEADERS);
  keymap_bind('f',  MENU_SEND, OP_SEND_EDIT_FCC);
  keymap_bind('t',  MENU_SEND, OP_SEND_EDIT_TO);
  keymap_bind('s',  MENU_SEND, OP_SEND_EDIT_SUBJECT);
  keymap_bind('\n', MENU_SEND, OP_SEND_SEND_MESSAGE);
  keymap_bind('\r', MENU_SEND, OP_SEND_SEND_MESSAGE);
  keymap_bind('y',  MENU_SEND, OP_SEND_SEND_MESSAGE);
  keymap_bind('n',  MENU_SEND, OP_SEND_ABORT_MESSAGE);
  keymap_bind('P',  MENU_SEND, OP_SEND_POSTPONE_MESSAGE);
#ifdef _PGPPATH
  keymap_bind('p',  MENU_SEND, OP_SEND_PGP_MENU);
#endif
#ifdef ISPELL
  keymap_bind('i',  MENU_SEND, OP_SEND_ISPELL);
#endif
  keymap_bind('v',  MENU_SEND, OP_SEND_ALIAS_MENU);

  generic_bindings(MENU_POST);
  keymap_bind('d', MENU_POST, OP_GENERIC_DELETE_ENTRY);
  keymap_bind('u', MENU_POST, OP_GENERIC_UNDELETE_ENTRY);

  generic_bindings(MENU_SENDATTACH);
  keymap_bind('d', MENU_SENDATTACH, OP_GENERIC_DELETE_ENTRY);
  keymap_bind('a', MENU_SENDATTACH, OP_SENDATTACH_ADD_ATTACHMENT);
  keymap_bind('s', MENU_SENDATTACH, OP_SENDATTACH_ENTER_DESCRIPTION);

  /* These menus only use the generic keybindings. */
  generic_bindings(MENU_FOLDER);
  generic_bindings(MENU_ALIAS);
  generic_bindings(MENU_URL);
}

/*
 * most important, get function if you have the character and the menu
 */
int keymap_lookup (int ch, int menu) {
  KEYMAP *p = (ch < KEY_MIN) ? Keymap[ch] : Keymap_func[ch-KEY_MIN];
  
  while (p) {
    if (p->menu == menu) return(p->function);
    p = p->next;
  }
  return(0);
}
