/* Copyright (C) 1996 Michael R. Elkins.
 * 
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 * 
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */ 

#ifdef HAVE_NCURSES_H
# include <ncurses.h>
#else
# include <curses.h>
#endif

#include <ctype.h>

#define CLEARLINE(x) move(x,0), clrtoeol()
#define CENTERLINE(x,y) move(y, (COLS-strlen(x))/2), addstr(x)

#define ctrl(c) ((c)-'A'+1)

#ifdef NCURSES_MOUSE_VERSION
extern mmask_t Mousemask;
#endif /* NCURSES_MOUSE_VERSION */

int ci_getch (void);
int ci_parse_bind (char *, char *);
int ci_enter_string (char *, size_t, int, int, int);
int ci_get_field (char *, char *, size_t, int);

#define IsPrint(c) (isprint(c) || (((c) > 0xa0) && ((c) < 0xff)))

/* ----------------------------------------------------------------------------
 * Support for color
 */

#define MT_COLOR_HEADER     1
#define MT_COLOR_QUOTED     2
#define MT_COLOR_SIGNATURE  3
#define MT_COLOR_SUBJECT    4
#define MT_COLOR_FROM       5
#define MT_COLOR_TOOLBAR    6
#define MT_COLOR_INDICATOR  7
#define MT_COLOR_ATTACHMENT 8
#define MT_COLOR_STATUS     9
#define MT_COLOR_TREE       10
#define MT_COLOR_NORMAL     11
#define MT_COLOR_ERROR      12
#define MT_COLOR_MAX        13

struct color_def {
#ifdef HAVE_COLOR
  short fore;
  short back;
  short bright;
  int color; /* attribute for color terminals */
#endif
  int mono;   /* attributes for mono terminals */
};

#ifdef COLOR_C
struct color_def Color_defs[MT_COLOR_MAX];
#else
extern struct color_def Color_defs[MT_COLOR_MAX];
#endif

void ci_init_color (void);
void ci_start_color (void);

#ifdef HAVE_COLOR
#define SETCOLOR(X) attrset((has_colors() ? Color_defs[X-1].color : Color_defs[X-1].mono))
#else
#define SETCOLOR(X) attrset(Color_defs[X-1].mono)
#endif

/* ----------------------------------------------------------------------------
 * These are here to avoid compiler warnings with -Wall under SunOS 4.1.x
 */
#if !defined(STDC_HEADERS) && !defined(NCURSES_VERSION)
extern int endwin();
extern int printw();
extern int beep();
extern int isendwin();
extern int w32addch();
extern int keypad();
extern int wclrtobot();
extern int mvprintw();
extern int getcurx();
extern int getcury();
extern int noecho();
extern int wdelch();
extern int wrefresh();
extern int wmove();
extern int wclear();
extern int waddstr();
extern int wclrtoeol();
#endif
