/*
 * Copyright (C) 1996 Michael R. Elkins.
 * 
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 * 
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */ 

#ifndef MUTTLIB_H
#define MUTTLIB_H

#include "config.h"

#include <stdio.h>
#include <sys/types.h>
#include <time.h>
#include <limits.h>
#include <setjmp.h>
#include <stdarg.h>

#define TRUE 1
#define FALSE 0

#ifdef HAVE_SRAND48
#define LRAND lrand48
#define SRAND srand48
#define DRAND drand48
#else
#define LRAND rand
#define SRAND srand
#define DRAND (double)rand
#endif /* HAVE_SRAND48 */

#ifdef HAVE_SETEGID
#define SETGID setegid
#else
#define SETGID setgid
#endif

/* a (more) generic interface to regular expression matching */
#ifdef HAVE_REGCOMP
#ifdef HAVE_REGEX_H
#include <regex.h>
#endif
#define REGCOMP(X,Y) regcomp(&X, Y, REG_NOSUB)
#define REGEXEC(X,Y) regexec(&X, Y, (size_t)0, (regmatch_t *)0, (int)0)
#define REGFREE(X) regfree(&X)
#elif defined(HAVE_REGCMP)
#ifdef HAVE_LIBGEN_H
#include <libgen.h>
#endif
#define REGCOMP(X,Y) ((X = regcmp(Y, (char *)0)) == NULL)
#define REGEXEC(X,Y) regex(X,Y) == NULL
#define REGFREE(X) free(X)
#else
#define REGCOMP(X,Y) (re_comp(Y) != NULL)
#define REGEXEC(X,Y) !re_exec(Y)
#define REGFREE(X) 
#endif

#define whitespace(c) ((c) == ' ' || (c) == '\t' || (c) == '\r' || (c) == '\n')

#define FOREVER while (1)

#define MAGICMBOX   1
#define MAGICMMDF   2

/* The stream corresponding to the currently openend folder */
#define MAILFP Context->folder->fp

/* Content-Disposition values */
#define DISPINLINE 0
#define DISPATTACH 1

#define TYPE(X) body_types[(X)]
#define ENCODING(X) body_encodings[(X)]

#if DEBUG
#define dprint(N,X) if(debuglevel>=N) fprintf X
#else
#define dprint(N,X) 
#endif

#define strfcpy(A,B,C) strncpy(A,B,C), *(A+(C)-1)=0

#define HUGE_STRING	5120
#define LONG_STRING     1024
#define STRING          256
#define SHORT_STRING    128

/* Content-Type */
enum {
  TYPEOTHER=0,
  TYPEAUDIO,
  TYPEAPPLICATION,
  TYPEIMAGE,
  TYPEMESSAGE,
  TYPEMULTIPART,
  TYPETEXT,
  TYPEVIDEO
};

/* Content-Transfer-Encoding */
enum {
  ENCOTHER=0,
  ENC7BIT,
  ENC8BIT,
  ENCQUOTEDPRINTABLE,
  ENCBASE64,
  ENCBINARY
};

/* flags for mutt_copy_header() */
#define CH_UPDATE 1      /* update the status and x-status fields? */
#define CH_WEED   (1<<1) /* weed the headers? */
#define CH_DECODE (1<<2) /* do RFC1522 decoding? */
#define CH_XMIT   (1<<3) /* transmitting this message? */
#define CH_FROM   (1<<4) /* retain the "From " message separator? */
#define CH_PREFIX (1<<5) /* use Prefix string? */

enum {
  /* flags for mutt_open_mailbox() */
  M_NOSORT=1,/* do not sort the mailbox after opening it */

  /* flags for _mutt_make_string() */
  M_FORCESUBJ, /* print the subject even if it didn't change */
  M_NOTREE, /* don't draw the thread tree */
};

/* flags to ci_pattern_function() */
enum {
  ACTIONLIMIT=1,
  ACTIONSELECT,
  ACTIONFIND,
  ACTIONUNSELECT,
  ACTIONDELETE,
  ACTIONUNDELETE
};

/* flags to ci_send_message() */
#define SENDREPLY      1
#define SENDGROUPREPLY 2
#define SENDLISTREPLY  4
#define SENDFORWARD    8
#define SENDBATCH      16
#define SENDPOSTPONED  32

#define OPTPROMPTAFTER    1
#define OPTSTATUSONTOP    2
#define OPTXTERMMOUSE     3
#define OPTALLOW8BIT      4
#define OPTASKCOPY        5
#define OPTHOLD           6
#define OPTMETOO          7
#define OPTEDITHDRS       8
#define OPTARROWCURSOR    9
#define OPTASKCC          10
#define OPTHEADER         11
#define OPTPOINTNEW       12
#define OPTASKDEL         13
#define OPTASKMOVE        14
#define OPTREVALIAS       15
#define OPTFORCENAME      16
#define OPTKEEPEMPTY      17
#define OPTPAGERSTOP      18
#define OPTSIGDASHES      19
#define OPTASKPOSTPONE    20
#define OPTASKREPLYTO     21
#define OPTASKBCC         22
#define OPTAUTOINCLUDE    23
#define OPTAUTOEDIT       24
#define OPTASKPRINT       25
#define OPTPGPAUTOSIGN    26
#define OPTMARKOLD        27
#define OPTCONFIRMFILES   28
#define OPTCONFIRMCREATE  29
#define OPTCONFIRMAPPEND  30
#define OPTCONFIRMFOLDERS 31
#define OPTASKRECALL      32
#define OPTPOPDELETE      33

/* A dynamic array */
typedef struct {
  char **data;
  short len;
  short max;
} ARRAY;

typedef struct {
  FILE *fpin;
  FILE *fpout;
  char *prefix;
  int displaying; /* TRUE only if we are sending the output to a pager. */
#if 0
  ARRAY *array;
  char buffer[1024];
  short offset;
#endif
} STATE;

typedef struct address ADDRESS;
struct address {
  char *personal;
  char *mailbox;
  char *adl;
  char *host;
  char *error;
  ADDRESS *next;
};

typedef struct alias {
  char *name;
  ADDRESS *addr;
  struct alias *next;
} ALIAS;

typedef struct list_t {
  char *data;
  struct list_t *next;
} LIST;

typedef struct envelope {
  ADDRESS *from;
  ADDRESS *to;
  ADDRESS *cc;
  ADDRESS *bcc;
  ADDRESS *sender;
  ADDRESS *reply_to;
  char *subject;
  LIST *references;
  LIST *userhdrs;    /* user defined headers */
} ENVELOPE;

typedef struct userheader {
  char *key;
  char *val;
  struct userheader *next;
} USERHEADER;

typedef struct parameter PARAMETER;
struct parameter {
  char *attribute;
  char *value;
  PARAMETER *next;
};

/* Information that helps in determing the Content-* of an attachment */
typedef struct content {
  unsigned int space : 1; /* whitespace at the end of lines? */
  unsigned int binary : 1; /* long lines, or CR not in CRLF pair */
  unsigned int from : 1; /* has a line beginning with "From "? */
  unsigned int dot : 1; /* has a line consisting of a single dot? */
  long hibin; /* 8-bit characters */
  long lobin; /* unprintable 7-bit chars (eg., control chars) */
  long ascii;
  long linemax; /* length of the longest line in the file */
} CONTENT;

typedef struct body {
  unsigned int type : 3;
  unsigned int encoding : 3;
  unsigned int disposition : 1;
  unsigned int use_disp : 1; /* Content-Disposition field printed? */
  unsigned int unlink : 1;
  char *subtype;
  PARAMETER *parameter;
  char *description;
  long hdr_offset;
  long offset;
  long length;
  char *filename;
  CONTENT *content;
  struct body *next;
  struct body *parts; /* parts of a multipart message */
} BODY;

typedef struct header {
  unsigned int mime : 1;     /* has a Mime-Version header? */
  unsigned int metamail : 1; /* requires metamail? */
  unsigned int flagged : 1;  /* marked important? */
  unsigned int visible : 1;  /* matched the limit criteria? */
  unsigned int tagged : 1;
  unsigned int deleted : 1;
  unsigned int changed : 1;
  unsigned int old : 1;

  unsigned int read : 1;
  unsigned int replied : 1;
  unsigned int remail : 1; /* this message was bounced from someplace. */
#ifdef _PGPPATH
  unsigned int pgp : 2;
#endif
  unsigned int subject_changed : 1; /* used for threading */
  unsigned int xxx : 2;             /* unused */

  /* Components from the "DATE:" field. */
  unsigned int year : 7;
  unsigned int month : 5;
  unsigned int day : 6;
  unsigned int hours : 5;
  unsigned int minutes : 6;
  unsigned int seconds : 6;
  unsigned int zhours : 5;
  unsigned int zminutes : 6;
  unsigned int zoccident : 1;

  time_t received; /* time at which the message was placed in the mailbox */
  long offset; /* where in the stream does this message begin? */
  long length; /* how long is this message (headers + body)? */
  unsigned short lines; /* how many lines in the body of this message? */
  short index;          /* the absolute (unsorted) message number */
  short msgno;		/* number displayed to the user */
  short virtual;
  ENVELOPE *env;	/* envelope information */
  BODY *content;	/* list of MIME parts */
  char *message_id;
  
  /* the following are used for threading support */
  struct header *parent;
  struct header *child; /* decendants of this message */
  struct header *next;  /* next message in this thread */
  struct header *prev;  /* previous message in thread */
  char *tree;           /* character string to print thread tree */

} HEADER;

typedef struct {
  char *name;
  short magic;
  FILE *fp;
  time_t mtime; /* last modification date */
  long size;    /* size of the mailbox  */
} FOLDER;

typedef struct {
  FOLDER *folder;
  HEADER **hdrs;
  short hdrmax;
  short **v2r;             /* mapping from virtual to real msgno */
  short msgcount;          /* number of messages in the mailbox */
  short vcount;            /* the number of virtual messages */
  short tagged;            /* how many messages are tagged? */
  short new;               /* how many new messages? */
  unsigned int weed : 1;   /* is header weeding turned on? */
  unsigned int locked : 1;
} CONTEXT;

/*
 * Function prototypes.
 */

#define mutt_make_string(A,B,C,D) _mutt_make_string(A,B,C,D,M_NOTREE)
void _mutt_make_string (char *, size_t, char *, HEADER *, int);

FOLDER *folder_open(const char *, const char *mode);
void folder_close(FOLDER **);

int lock_file(FOLDER *, int);
void unlock_file(FOLDER *);

ADDRESS *mutt_default_from (void);
ADDRESS *mutt_expand_aliases (ADDRESS *);
ADDRESS *mutt_new_address (void);
ADDRESS *mutt_remove_duplicates (ADDRESS *);
BODY *mutt_make_attach (const char *);
BODY *mutt_new_body (void);
BODY *mutt_read_mime_header (FILE *);
BODY *mutt_make_attach (const char *);
BODY *mutt_make_multipart (BODY *);
ENVELOPE *mutt_new_envelope (void);
ENVELOPE *read_rfc822_header (FILE *, HEADER *);
HEADER *mutt_new_header (void);
LIST *mutt_new_list (void);
PARAMETER *mutt_new_parameter (void);
STATE *mutt_new_state (void);

char *mutt_generate_boundary (void);
char *mutt_get_parameter (const char *, PARAMETER *);
char *mutt_make_date (char *);
char *mutt_strlower (char *);

int mutt_addr_is_user (ADDRESS *);
int mutt_is_mail_list (ADDRESS *);
int is_local_site (const char *);

CONTEXT *mutt_open_mailbox (char *, CONTEXT *, int);
int mutt_check_mailbox (CONTEXT *);
int mutt_parse_mailbox (CONTEXT *);
int mutt_sync_mailbox (CONTEXT *);
int mutt_close_mailbox (CONTEXT *);
void mutt_sort_headers (CONTEXT *);
void mutt_fastclose_mailbox (CONTEXT *);

void mutt_add_to_list (LIST **, char *);
void mutt_block_signals (void);
void mutt_body_handler (BODY *, STATE *);
void mutt_bounce_message (HEADER *, char *);
void mutt_complete (char *);
void mutt_copy_bytes (FILE *, FILE *, long);
void mutt_copy_header (FILE *, HEADER *, FILE *, int);
void mutt_decode_base64 (STATE *, long, int);
void mutt_decode_quoted (STATE *, long, int);
void mutt_decode_xbit (STATE *, long);
void mutt_default_savename (char *, size_t, ENVELOPE *);
void mutt_do_thread (HEADER *, int);
void mutt_error (const char *, ...);
void mutt_exit (int);
void mutt_expand_path (char *);
void mutt_execute_macro (int, int);
void mutt_fetchPopMail(void);
void mutt_folder_cc (const char *, const char *);
void mutt_free_address (ADDRESS **);
void mutt_free_body (BODY **);
void mutt_free_envelope (ENVELOPE **);
void mutt_free_header (HEADER **);
void mutt_free_list (LIST **);
void mutt_free_state (STATE **);
void mutt_free_parameter (PARAMETER **);
void mutt_generate_header (char *, size_t, HEADER *, int);
void mutt_global_init (void);
void mutt_mktemp (char *);
void mutt_parse_mime_message (HEADER *);
void mutt_parse_references (LIST **, char *);
void mutt_pipe_message (HEADER *, const char *);
void mutt_postpone_message (const char *, ENVELOPE *, const char *, BODY *);
void mutt_print_message (HEADER *);
void mutt_show_version (void);
void mutt_simple_address (char *, size_t, ADDRESS *);
void mutt_sort_threads (CONTEXT *);
void mutt_status_line (char *, size_t);
void mutt_tabs_to_spaces (char *);
void mutt_text_handler (BODY *, STATE *);
void mutt_unblock_signals (void);
void mutt_unlink (const char *);
void mutt_write_address (char *, size_t, ADDRESS *);

int mutt_check_key (const char *);
int mutt_check_menu (const char *);
int mutt_check_mime_type (const char *);
int mutt_check_month (const char *);
int mutt_compat_charset (const char *);
int mutt_create_filter (const char *, FILE **, FILE **);
int mutt_get_password (char *, char *, size_t);
int mutt_init (int);
int mutt_is_text_type (int, char *);
int mutt_is_valid_mailbox (const char *);
int mutt_matches_ignore (const char *);
int mutt_needs_metamail (BODY *);
int mutt_next_thread (HEADER *);
int mutt_num_postponed (void);
int mutt_parse_adrlist (ADDRESS **, char *, char *);
int mutt_parse_macro (char *, char *, size_t);
int mutt_parse_rc_line (char *, int, char *, int);
int mutt_previous_thread (HEADER *);
int mutt_save_attachment (BODY *, char *);
int mutt_save_message (HEADER *, char *, int);
int mutt_send_message (ENVELOPE *, BODY *, const char *, int);
int mutt_system (const char *);
int mutt_was_modified (CONTEXT *);
int mutt_write_mime_body (BODY *, FILE *);
int mutt_write_mime_header (BODY *, FILE *);
int mutt_write_rfc822_header (FILE *, ENVELOPE *, BODY *, int);
int mutt_yesorno (const char *, int);

int option (int);

int state_puts (char *, STATE *);
int state_putc (int, STATE *);

char *safe_strdup (const char *);
void *safe_malloc (unsigned int);
void safe_realloc (void **, unsigned int);
void safe_free (void **);

char *strtokq (char *, char *);
char *skip_whitespace (char *);

/* from.c */
time_t is_from (const char *, char *, size_t);

/* aliaslib.c */
ADDRESS *alias_reverse_lookup (ADDRESS *);

/* parse.c */
BODY *parse_multipart (FILE *, char *);
BODY *parse_messageRFC822 (FILE *, BODY *);
BODY *read_mime_header (FILE *);

/* pgp.c */
#ifdef _PGPPATH
#define PGPENCRYPT 1
#define PGPSIGN    2
#define PGPKEY     3

int pgp_valid_passphrase (void);
void application_pgp_handler (BODY *, STATE *);
void pgp_signed_handler (BODY *, STATE *);
void pgp_encrypted_handler (BODY *, STATE *);
BODY *pgp_signMessage (BODY *);
BODY *pgp_encryptMessage (BODY *, char *, int);
char *pgp_findKeys (ADDRESS *, ADDRESS *, ADDRESS *);
int pgp_sendMenu (void);
int pgp_query (BODY *);
void pgp_void_passphrase (void);
#endif /* _PGPPATH */

#ifndef HAVE_SNPRINTF
extern int snprintf (char *, size_t, const char *, ...);
#endif

/*
 * AIX doesn't define these in <string.h>.  These prototypes avoid compiler
 * warnings with "gcc -Wall" and should be harmless on other systems.
 */
int strcasecmp (const char *, const char *);
int strncasecmp (const char *, const char *, size_t);

#ifndef STDC_HEADERS
extern FILE *fdopen();
extern int system();
extern int puts();
extern int fputs();
extern int fputc();
extern int fseek();
extern char *strchr();
extern int getopt();
extern int fputs();
extern int fputc();
extern int fclose();
extern int fprintf();
extern int printf();
extern int fgetc();
extern int tolower();
extern int toupper();
extern int sscanf();
extern size_t fread();
extern size_t fwrite();
extern int system();
extern int rename();
extern time_t time();
extern struct tm *localtime();
extern char *asctime();
extern char *strpbrk();
extern int fflush();
extern long lrand48();
extern void srand48();
extern time_t mktime();
extern int vsprintf();
extern int ungetc();
extern char *mktemp();
extern int ftruncate();
extern char *re_comp();
extern int re_exec();
extern void *memset();
extern int pclose();
#endif

/* nifty trick I stole from ELM 2.5alpha. */
#ifdef MAIN_C
#define WHERE 
#define INITVAL(x) = x
#else
#define WHERE extern
#define INITVAL(x) 
#endif

WHERE jmp_buf Jumpbuf;

WHERE CONTEXT *Context;

WHERE char Errorbuf[SHORT_STRING];
WHERE char Init_error[SHORT_STRING];

WHERE char AliasFile[_POSIX_PATH_MAX];
WHERE char Attribution[SHORT_STRING];
WHERE char Charset[SHORT_STRING];
WHERE char Domain[SHORT_STRING];
#ifdef USE_DSN
WHERE char DsnNotify[SHORT_STRING] INITVAL("");
WHERE char DsnReturn[SHORT_STRING] INITVAL("");
#endif
WHERE char Editor[SHORT_STRING];
WHERE char Fqdn[SHORT_STRING];
WHERE char Hdr_format[SHORT_STRING];
WHERE char Homedir[_POSIX_PATH_MAX];
WHERE char Hostname[SHORT_STRING];
WHERE char In_reply_to[SHORT_STRING];
WHERE char Inbox[_POSIX_PATH_MAX];
WHERE char Local_sig[_POSIX_PATH_MAX] INITVAL({0});
WHERE char Maildir[_POSIX_PATH_MAX];
WHERE char Metamail[_POSIX_PATH_MAX];
WHERE char Muttrc[_POSIX_PATH_MAX];
WHERE char Outbox[_POSIX_PATH_MAX] INITVAL({0});
WHERE char Pager[_POSIX_PATH_MAX];
WHERE char Pager_fmt[SHORT_STRING];
WHERE char Parse_error[SHORT_STRING] INITVAL({0});
WHERE char Pgp[_POSIX_PATH_MAX];
WHERE char PopHost[SHORT_STRING] INITVAL({0});
WHERE char PopUser[SHORT_STRING] INITVAL({0});
WHERE char PopPass[SHORT_STRING] INITVAL({0});
WHERE char PostIndentString[SHORT_STRING] INITVAL({0});
WHERE char Postponed[_POSIX_PATH_MAX];
WHERE char Prefix[SHORT_STRING];
WHERE char Print[_POSIX_PATH_MAX];
WHERE char QuoteString[SHORT_STRING];
WHERE char Realname[SHORT_STRING];
WHERE char Remote_sig[_POSIX_PATH_MAX] INITVAL({0});
WHERE char Sendmail[_POSIX_PATH_MAX];
WHERE char Shell[_POSIX_PATH_MAX];
WHERE char Signature[_POSIX_PATH_MAX];
WHERE char Spoolfile[_POSIX_PATH_MAX];
WHERE char Status[SHORT_STRING];
WHERE char Tochars[SHORT_STRING];
WHERE char Tempdir[_POSIX_PATH_MAX];
WHERE char Username[SHORT_STRING];
WHERE char Web_browser[_POSIX_PATH_MAX] INITVAL({0});
WHERE char Web_Xbrowser[_POSIX_PATH_MAX] INITVAL({0});

WHERE LIST *Ignore INITVAL(0);
WHERE LIST *Mailing_lists INITVAL(0);

/* bit vectors for boolean variables */
WHERE unsigned long OptionsA INITVAL(0);
WHERE unsigned long OptionsB INITVAL(0);

WHERE short Default_magic INITVAL(MAGICMBOX);
WHERE short ReadMsginc INITVAL(10);
WHERE short WriteMsginc INITVAL(10);
WHERE short Sleep_time INITVAL(1);
WHERE short Sort;
WHERE short Timeout INITVAL(600); /* seconds */

WHERE ALIAS *Aliases INITVAL(0);
WHERE ADDRESS *Alternates INITVAL(0);
WHERE USERHEADER *User_defined_headers INITVAL(0);
WHERE LIST *Localsites INITVAL(0);

#if DEBUG
WHERE FILE *debugfile INITVAL(0);
WHERE int debuglevel INITVAL(0);
#endif

#ifdef USE_SETGID
WHERE gid_t Mail_gid;
WHERE gid_t User_gid;
#endif /* USE_SETGID */

#ifdef MAIN_C
const char *Weekdays[] = { "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" };
const char *Months[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec", "ERR" };

const char *body_types[] = { "x-unknown", "audio", "application", "image", "message", "multipart", "text", "video" };
const char *body_encodings[] = { "x-unknown", "7bit", "8bit", "quoted-printable", "base64", "binary" };
#else
extern const char *Weekdays[];
extern const char *Months[];
extern const char *body_types[];
extern const char *body_encodings[];
extern const char Mutt_version[];
#endif

#endif /* MUTTLIB_H */
