/*
 * Copyright (C) 1996 Michael R. Elkins.
 * 
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 * 
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */ 

#include "mutt.h"
#include "mutt_curses.h"

#include <string.h>

enum { BCC=1, CC, FROM, SUBJECT, TO };

enum {
  FLAG_ALL=1,
  FLAG_TAGGED,
  FLAG_NEW,
  FLAG_UNREAD,
  FLAG_FLAGGED,
  FLAG_THREAD
};

/* Returns true if string "a" is found in string "b" */
static int pattern_match (char *a, char *b)
{
  int len = strlen(a);
  
  while (*b) {
    if (strncasecmp(a, b, len) == 0) return 1;
    b++;
  }
  return 0;
}

static int meta_match (int f, ENVELOPE *h, char *c)
{
  char buffer[SHORT_STRING];
  ADDRESS *a;
  
  switch (f) {
  case SUBJECT:
    return (h->subject ? pattern_match(c, h->subject) : 0);
  case FROM:
    a = h->from;
    break;
  case TO:
    a = h->to;
    break;
  case CC:
    a = h->cc;
    break;
  default:
    return 0;
  }
  buffer[0] = 0;
  mutt_write_address(buffer, sizeof(buffer), a);
  return(pattern_match(c, buffer));
}

#define DELETE(x) Context->hdrs[x]->deleted=1
#define UNDELETE(x) Context->hdrs[x]->deleted=0

static void tag_message (int n)
{
  if (!Context->hdrs[n]->tagged) {
    Context->hdrs[n]->tagged = 1;
    Context->tagged++;
  }
}

static void untag_message (int n)
{
  if (Context->hdrs[n]->tagged) {
    Context->hdrs[n]->tagged = 0;
    Context->tagged--;
  }
}

static void make_visible (int n)
{
  Context->hdrs[n]->visible = 1;
  Context->hdrs[n]->virtual = Context->vcount;
  *Context->v2r[Context->vcount] = n;
  Context->vcount++;
}
  
static void do_action (short op, short count, short **vec)
{
  short i;

  for (i=0; i < Context->msgcount; i++) {
    if (*vec[i] == count) {
      switch (op) {
      case ACTIONLIMIT:
	make_visible(i);
	break;
      case ACTIONSELECT:
	tag_message(i);
	break;
      case ACTIONUNSELECT:
	untag_message(i);
	break;
      case ACTIONDELETE:
	DELETE(i);
	break;
      case ACTIONUNDELETE:
	UNDELETE(i);
	break;
      }
    }
  }
}

static int matches_flag (HEADER *h, int flag)
{
  switch (flag) {
  case FLAG_ALL:
    return 1;
  case FLAG_TAGGED:
    return(h->tagged);
  case FLAG_NEW:
    return(! (h->old || h->read));
  case FLAG_UNREAD:
    return(!h->read);
  case FLAG_FLAGGED:
    return(h->flagged);
  default:
    return 0;
  }
  /* not reached */
}

/*
 * A generic function to delete/undelete/tag/untag/limit/unlimit messages
 * which match a patter inputted by the user.
 * return 0 if something happened, -1 otherwise
 */
int ci_pattern_function (int op, char *prompt, HEADER *cur)
{
  char *crit, buffer[SHORT_STRING] = { 0 };
  int i, field=0, count=0;
  short **vec = 0;
  int rc = 0;
  int flag = 0;

  if (ci_get_field(prompt, buffer, sizeof(buffer), 0) != 0 || !buffer[0])
    return(-1);

  if (strcasecmp(buffer, "all") == 0)
    flag = FLAG_ALL;
  else if (strcasecmp(buffer, "new") == 0)
    flag = FLAG_NEW;
  else if (strcasecmp(buffer, "tagged") == 0)
    flag = FLAG_TAGGED;
  else if (strcasecmp(buffer, "unread") == 0)
    flag = FLAG_UNREAD;
  else if (strcasecmp(buffer, "flagged") == 0)
    flag = FLAG_FLAGGED;
  else if (strcasecmp(buffer, "thread") == 0)
    flag = FLAG_THREAD;
  else
    flag = 0;

  if (op == ACTIONLIMIT) {
    Context->vcount = 0;
    for (i=0; i<Context->msgcount; i++) {
      Context->hdrs[i]->visible = 0;
      Context->hdrs[i]->virtual = -1;
    }
  }
      
  if (flag == FLAG_THREAD)
    mutt_do_thread(cur, op);
  else if (flag) {
    for (i = 0; i < Context->msgcount; i++) {
      if (matches_flag(Context->hdrs[i], flag)) {
	switch (op) {
	case ACTIONDELETE:
	  DELETE(i);
	  break;
	case ACTIONUNDELETE:
	  UNDELETE(i);
	  break;
	case ACTIONSELECT:
	  tag_message(i);
	  break;
	case ACTIONUNSELECT:
	  untag_message(i);
	  break;
	case ACTIONLIMIT:
	  make_visible(i);
	  break;
	}
      }
    }
  }
  else {
    /*
     * Create a "collision" vector to take care of AND'ing terms.  At the
     * end we compare how many hits each message got.  If it is not equal to
     * the total number of searches, then it did not match all given
     * criteria.
     */
    vec = (short **)safe_malloc(sizeof(short *) * Context->msgcount);
    for (i=0; i<Context->msgcount; i++) {
      vec[i] = (short *)safe_malloc(sizeof(short));
      *vec[i] = 0;
    }

    crit = buffer;
    while ((crit = strtokq(crit, " \t")) != 0) {
      
      if (*crit == '|') {	
	do_action(op, count, vec);
	
	count = 0;
	for (i=0; i<Context->msgcount; i++)
	  *vec[i] = 0;
	
	if (! *(crit+1)) {
	  crit=0;
	  continue;
	}
	else
	  crit = skip_whitespace(crit+1);
      }
      
      count++; /* keep track of how many searches we have done. */
      
      if (*crit == '~') {
	/* acccpt DejaNews-style search patterns. */
	switch(*(crit+1)) {
	case 's':
	  field = SUBJECT;
	  break;
	case 't':
	  field = TO;
	  break;
	case 'c':
	  field = CC;
	  break;
	case 'f':
	  field = FROM;
	  break;
	default:
	  mutt_error("Unknown search key: %s", crit);
	  rc = -1;
	  goto free_mem;
	}
      }
      else if (tolower(*crit) == 'f') {
	if (strcasecmp("rom", crit+1) == 0) {
	  field = FROM;
	}
      }
      else if (tolower(*crit) == 's') {
	if (strcasecmp("ubject", crit+1) == 0 || strcasecmp("ubj", crit+1) == 0)
	  field = SUBJECT;
      }
      else if (tolower(*crit) == 'c') {
	if (strcasecmp("c", crit+1) == 0)
	  field = CC;
      }
      else if (tolower(*crit) == 't') {
	if (strcasecmp("o", crit+1) == 0)
	  field = TO;
      }
      else if (isdigit(*crit)) {
	int max, min;
	
	/* this must be a range of messages. */
	if (sscanf(crit, "%d-%d", &min, &max) != 2 || min < 1 || max > Context->msgcount) {
	  mutt_error("Invalid message range.");
	  rc = -1;
	  goto free_mem;
	}
	
	for (i=min-1; i<max; i++) {
	  (*vec[i])++;
	  do_action(op, 1, vec);
	}
	
	goto free_mem;
      }
      else {
	mutt_error("Invalid search key.");
	rc = -1;
	goto free_mem;
      }
      
      if ((crit = strtokq(NULL, " \t")) == NULL) {
	mutt_error("Missing argument!");
	rc = -1;
	goto free_mem;
      }

      for (i=0; i < Context->msgcount; i++) {
	if (meta_match(field, Context->hdrs[i]->env, crit))
	  (*vec[i])++; /* increase the hit count for this message. */
      }
      
      crit = 0;
    }

    do_action(op, count, vec);

  free_mem:
    for (i=0; i<Context->msgcount; i++)
      safe_free((void **)&vec[i]);
    safe_free((void **)&vec);
  }

  if (op == ACTIONLIMIT && !Context->vcount) {
    mutt_error("No messages matched!");

    /* reset the virtual message numbers */
    for (i = 0; i < Context->msgcount; i++) {
      *Context->v2r[i] = i;
      Context->hdrs[i]->virtual = i;
    }
    Context->vcount = Context->msgcount;
  }

  return(rc);
}
