/*
 * Copyright (C) 1996 Michael R. Elkins <me@cs.hmc.edu>
 * 
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 * 
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */ 

#include "muttlib.h"
#include "rfc1522.h"
#include "rfc_gen.h"

#include <ctype.h>
#include <string.h>

/* pms@romance.kaist.ac.kr */
#define IsPrint(c) (isprint(c) || (((c) > 0xa0) && ((c) < 0xff)))

/*
**  Header Q-encoding - pms@romance.kaist.ac.kr
*/
void rfc1522_Q_encode_string (char *d, unsigned char *s, int l)
{
  unsigned char *e=d;
  char charset[32];
  register int i, count, nel, ewl, ewl0, ewl1, csl, ch8;

  /* First check to see if there are any 8-bit characters */
  for(i=count=nel=0; !count && i<strlen(s); i++) {
    if (s[i] & 0x80) count++;
    else if (count == 0 && s[i] == ' ') nel = i;
  }
  if (!count) { 
    strfcpy(d, s, l); 
    return;
  }

  /* Generate an appropriate RFC1522 word */
  strcpy(charset, 
    strcasecmp("US-ASCII", Charset) == 0 ? "UNKNOWN-8BIT" : Charset);
  csl = strlen(charset);
  /* Copy the first portion of the string that doesn't need to be encoded. */
  if(nel) {
     for(i=0; i<nel+1; i++) *e++ = s[i];
     s += nel + 1;
  }
  /* RFC1522 states that a line should not contain more than 76 characters. */
  ewl0 = 67 - 13 - nel - csl - 2;
  ewl1 = 67 - csl - 2;

  for(count=0; *s; ) { 
    unsigned char ob[STRING] ="";
    ewl = (count)? ewl1: ewl0;
    for(i=ch8=0; *s && i<ewl; i++, s++) {
      /* Encode all 8-bit characters plus any characters that would cause the
       * string to be enclosed in quotes ("") by the rfc822_*() routines.
       * Encoded words are not legal inside of quotes ("") according to
       * RFC1522.
       */
      if ((*s) & 0x80 || strchr("?=_.[]()\\\"@<>/,;:", *s) != 0) {
        sprintf(ob+i, "=%2.2X", *s);
        i += 2;
      }
      else if(*s==' ') ob[i] = '_';
      else ob[i] = *s;
      ch8 = ((*s) & 0x80)? ch8++: 0;
    }
    /* Do not cut a double-byte pair at the border */
    if(ch8%2) { ob[i-3] = 0; --s; }
    else ob[i] = 0;
    sprintf(e, "=?%s?Q?%s?=", charset, ob);
    e += strlen(ob) + 7 + csl;
    if (*s) {
      sprintf(e, "\n ");
      e += 2;
      count++;
    }
  }
  *e = 0;
}

/*
**  Consider nation specific RFCs - pms@romance.kaist.ac.kr
*/
void rfc1522_encode_string(char *d, unsigned char *s, int l)
{
  switch(rfc_own_header_coding()) {
    case B_HEADER:
     rfc1522_B_encode_string(d, s, l);
     break;
    case Q_HEADER:
    default:
     rfc1522_Q_encode_string(d, s, l);
     break;
  }
}

void rfc1522_encode_address (ADDRESS *addr)
{
  ADDRESS *ptr = addr;
  char buffer[STRING];

  while (ptr) {
    if (ptr->personal) {
      rfc1522_encode_string (buffer, (unsigned char *)&ptr->personal, sizeof(buffer));
      safe_free((void **)&ptr->personal);
      ptr->personal = safe_strdup(buffer);
    }
    ptr = ptr->next;
  }
}

/*
** More allowable header decoding - pms@romance.kaist.ac.kr
*/
static int rfc1522_decode_word (char *d, const char *s, int len)
{
  char *p = safe_strdup(s);
  char *pp = p;
  char *pd = d;
  int enc = 0, filter = 0, count = 0;

  while ((pp = strtok (pp, "?")) != NULL) {
    count++;
    switch (count) {
    case 2:
	  filter = 1;
      break;
    case 3:
      if (toupper(*pp) == 'Q')
        enc = ENCQUOTEDPRINTABLE;
      else if (toupper(*pp) == 'B')
        enc = ENCBASE64;
      else
        return (-1);
      break;
    case 4:
      if (enc == ENCQUOTEDPRINTABLE) DecodeQP(pd, pp);
      else if (enc == ENCBASE64) DecodeBase64(pd, pp);
      break;
    }
    pp = 0;
  }
  safe_free ((void **)&p);
  if (filter) {
    pd = d;
    while (*pd) {
      if (!IsPrint((unsigned char)*pd)) *pd = '?';
      pd++;
    }
  }
  return (0);
}

/* Gack!  Parsing RFC822 structured fields is a royal pain... */
static int rfc822_toklen (char *s)
{
  char *p = s;
  short depth;

  while (p) {
    if (*p == '(') {
      depth = 0;
      while (*p) {
        if (*p == '\\')
	  p++;
        else if (*p == '\"')
	  p += rfc822_toklen(p);
        else if (*p == '(')
	  depth++;
        else if (*p == ')')
	  depth--;
        p++;
        if (depth == 0)
	  break;
      }
      break;
    }
    else if (*p == '\"') {
      p++;
      while (*p) {
        if (*p == '\\')
	  p++;
        else if (*p == '\"')
	  break;
	p++;
      }
      break;
    }
    else if (strchr(".:@ \t", *p) != 0)
      break;
    p++;
  }
  return(p != s ? p-s : 1);
}

static int comment_toklen (char *s)
{
  char *p = strpbrk(s, " \r\n\t(");
  if (p) return (p != s ? p-s : 1);
  else return(strlen(s));
}

static int simple_toklen(char *s)
{
  char *p = strpbrk(s, " \r\n\t");
  if (p) return (p != s ? p-s : 1);
  else return(strlen(s));
}

#define IN_STRUCTURED 1
#define IN_COMMENT    2
#define IN_PHRASE     4

int rfc1522_decode_real (char *d, char *s, int len, int flag)
{
  int toklen, wordlen;
  char *ps, savechar, *pd = d;
  
  while (*s && (len > 0)) {
    if (flag & IN_STRUCTURED)
      toklen = rfc822_toklen(s);
    else if (flag & IN_COMMENT)
      toklen = comment_toklen(s);
    else
      toklen = simple_toklen(s);
   
    /* 
    ** Delimit encoded quotes - pms@romance.kaist.ac.kr 
    **    see 'rfc1522_decode' func. and 'parse.c' 
    */
    if(*s=='\t' && *(s+1)==' ') { s+=2; continue; }
    ps = s + toklen;
    savechar = *ps;
    *ps = 0;
    
    if (((flag & IN_STRUCTURED) || (flag & IN_COMMENT)) && *s == '(') {
      *pd++ = '(';
      len--;
      s[toklen-1] = 0;
      if (rfc1522_decode_real (pd, s+1, len, IN_COMMENT) == -1) return(-1);
      wordlen = strlen(pd);
      pd += wordlen;
      len -= wordlen;
      *pd++ = ')';
      len--;
    }
    else if ((((flag & IN_STRUCTURED) && (flag & IN_PHRASE)) ||
        !(flag & IN_STRUCTURED)) &&
       (s[0] == '=' && s[1] == '?' && s[toklen-2] == '?' && s[toklen-1] == '=')) {
      
      if (rfc1522_decode_word(pd, s, len) == -1) return(-1);
      wordlen = strlen(pd);
      pd += wordlen;
      len -= wordlen;
    }
    else {
      len -= toklen;
      if (len < 0) return(-1);
      strncpy(pd, s, toklen);
      pd += toklen;
    }

    *ps = savechar;
    s = ps;
  }
  *pd = 0;
  return(0);
}

int rfc1522_valid (const char *s)
{
  char *p = strstr(s, "=?");
  if (!p) return(0);
  p = strstr(p+2, "?=");
  if (!p) return(0);
  return(1);
}

#define HDR_STRUCTURED 1
#define HDR_PHRASE     2

/*
** More allowable decoding - pms@romance.kaist.ac.kr 
*/
void rfc1522_decode (char *s, int len, int flags)
{
  char buffer[LONG_STRING], *p;
  int passflags = 0;

  if (!rfc1522_valid(s)) {
    /*
    ** This is required, I modified 'parse.c' to follow RFC1522 strictly.
    **    See also 'rfc1522_decode_real' func. - pms@romance.kaist.ac.kr
    */
    p = strstr(s, "\t ");
    if(p) strcpy(p, p+1);
    return; /* nothing to do */
  }
  strncpy(buffer, s, sizeof(buffer));
  if (flags & HDR_STRUCTURED) {
    passflags = IN_STRUCTURED;
    if (flags & HDR_PHRASE)
      passflags |= IN_PHRASE;
  }
  if (rfc1522_decode_real(s, buffer, len, passflags) == -1)
    strcpy(s, buffer);
  if(rfc1522_valid(s)) {
    strcpy(buffer, s); s[0] = 0;
    DecodeHeader(s, buffer);
  }
}

/*
 * This piece of code, and the idea of using the HDR_* flags, is attributed
 * to Kari Hurtta <kari.hurtta@fmi.fi>, who helped me get this right.
 */
int rfc1522_classify (const char *s)
{
  if (strncasecmp("from:", s, sizeof("from:")-1) == 0 ||
      strncasecmp("to:", s, sizeof("to:")-1) == 0 ||
      strncasecmp("cc:", s, sizeof("cc:")-1) == 0 ||
      strncasecmp("bcc:", s, sizeof("bcc:")-1) == 0)
    return(HDR_STRUCTURED | HDR_PHRASE);
  return(0);
}

void rfc1522_decode_adrlist (ADDRESS *a)
{
  while (a) {
    if (a->personal && rfc1522_valid(a->personal)) {
      rfc1522_decode(a->personal, strlen(a->personal), 0);
      safe_realloc((void **)&a->personal, strlen(a->personal)+1);
    }
    a = a->next;
  }
}

void rfc1522_encode_adrlist (ADDRESS *p)
{
  char buffer[LONG_STRING];

  while (p) {
    if (p->personal) {
      rfc1522_encode_string(buffer, (unsigned char *)p->personal, sizeof(buffer)-1);
      safe_free((void **)&p->personal);
      p->personal = safe_strdup(buffer);
    }
    p = p->next;
  }
}
