/* Copyright (C) 1996 Michael R. Elkins.
 * 
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 * 
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */ 

#define TOOLBAR_C

#include "mutt.h" /* for definition of WHERE */
#include "mutt_curses.h"
#include "toolbar.h"
#include "keymap.h"

#include <stdlib.h>
#include <string.h>

#ifdef NCURSES_MOUSE_VERSION

TOOLBAR Generic_toolbar[] = {
  { "Quit",     OP_GENERIC_EXIT,          0 },
  { "Top",      OP_GENERIC_FIRST_ENTRY,   0 },
  { "Bottom",   OP_GENERIC_LAST_ENTRY,    0 },
  { "NextPage", OP_GENERIC_NEXT_PAGE,     0 },
  { "PrevPage", OP_GENERIC_PREVIOUS_PAGE, 0 },
  { 0, 0, 0 }
};

/*
 * This function automatically lays out the buttons on the toolbar.
 */
void toolbar_init (TOOLBAR *t) {
  int i;

  for (i = 1; t[i-1].button; i++) {
    t[i].offset = t[i-1].offset + strlen(t[i-1].button) + 1;
  }
}

/*
 * Write the toolbar to the screen.
 */
void toolbar_post (TOOLBAR *t) {
  int i;
  char buffer[256];

  buffer[0] = 0;
  for (i = 0; t[i].button; i++)
    sprintf(buffer+strlen(buffer), "%s ", t[i].button);
  move(0,0);
  clrtoeol();
  SETCOLOR(MT_COLOR_TOOLBAR);
  printw("%-*.*s", COLS, COLS, buffer);
  SETCOLOR(MT_COLOR_NORMAL);
}

/*
 * Returns the operation associated with the button at location 'x'
 */
int toolbar_select (TOOLBAR *t, int x) {
  int i;

  for (i=1; t[i-1].button; i++)
    if (x < t[i].offset) return(t[i-1].op);
  return(-1);
}

#endif /* NCURSES_MOUSE_VERSION */
